% This function handles the common geometry creation tasks, outsourcing
% the actual structure specific tasks to external functions to draw the
% slab/holes.
% Inputs are:
%
% fem = optical FEM structure 
% P = structure containing geometry information specific to the unit cell in
%     question; see associated generation files for information; fields
%     which are necessary for this function are:
%       P.type = string denoting the type of unit cell to be drawn
%       P.useSym = 3 element vector denoting whether symmetry planes are used;
%       P.thi = slab thickness;
%       P.air = thickness of air slab (if used)
%       P.rFillet = fillet radius for snowflake corners (optional)
%
% varargin = additional arguments, submitted as 'variableName',value pairs;
%            supported arguments are:
%               useAir = whether or not to add an external air space
%                        (optical vs. mechanical sims)
%
% Sean Meenehan, 10/29/13

function [fem,P,latticeVecs,fixedBounds] = createUnitCell(fem,P,varargin)

% Parse varargin
useAir = 0;
if(mod(length(varargin),2)>0); error('Incorrect parameter specification.'); end;
for k=1:2:length(varargin)
    if (exist(varargin{k},'var'))
        eval([varargin{k},' = ',num2str(varargin{k+1}),';']);
    else 
        display([varargin{k} ' is not a valid parameter.']);
    end
end

useSym = P.useSym;
thi = P.thi;

% globals
global CORNER_FILLET_RADIUS;
CORNER_FILLET_RADIUS = 0;
if isfield(P,'rFillet')
    CORNER_FILLET_RADIUS = P.rFillet;
end

%% Select cell type. To add new cell type, create an appropriate generate* function and add here
if isfield(P,'pG')
    % Information from patch geometry is generated using patchGeom. 
    slabInfo = P.pG.slabInfo;
    holeInfo = P.pG.holeInfo;
    latticeVecs = P.pG.latticeVecs;
    fixedBounds = P.pG.fixedBounds;
elseif strcmp(P.type,'Nanobeam')
    [slabInfo,holeInfo,latticeVecs,fixedBounds] = generateUnitCellNanobeam(P);
elseif strcmp(P.type,'Cross')
	[slabInfo,holeInfo,latticeVecs,fixedBounds] = generateUnitCellCross(P);
elseif strcmp(P.type,'Snowflake')
	[slabInfo,holeInfo,latticeVecs,fixedBounds] = generateUnitCellSnowflake(P);
elseif strcmp(P.type,'SnowflakeW1')
	[slabInfo,holeInfo,latticeVecs,fixedBounds] = generateUnitCellSnowflakeW1(P);
elseif strcmp(P.type,'SnowflakeW1RMod')
	[slabInfo,holeInfo,latticeVecs,fixedBounds] = generateUnitCellSnowflakeW1RMod(P);
elseif strcmp(P.type,'SnowflakeW1Ellipse')
	[slabInfo,holeInfo,latticeVecs,fixedBounds] = generateUnitCellSnowflakeW1Ellipse(P);
elseif strcmp(P.type,'SnowflakeW1EllipseCenter')
	[slabInfo,holeInfo,latticeVecs,fixedBounds] = generateUnitCellSnowflakeW1EllipseCenter(P);
else
	error(['Unrecognized cavity type: ',P.type]);
end

%% Draw slabs
slabs = {};
airSlab = [];
for k = 1:length(slabInfo)
    if (strcmp(slabInfo{k}.type,'Rectangle'))
		if (slabInfo{k}.wx > 1e-9 && slabInfo{k}.wy > 1e-9)
			s = rect2(slabInfo{k}.wx,slabInfo{k}.wy,'base','center','pos',[slabInfo{k}.x slabInfo{k}.y]);
            slabs{end+1} = s;
        end	
    elseif (strcmp(slabInfo{k}.type,'Polygon'))
        xVec = [];
        yVec = [];
        for vert = 1:length(slabInfo{k}.x)
            xVec(end+1) = slabInfo{k}.x(vert);
            yVec(end+1) = slabInfo{k}.y(vert);
        end
        s = line2(xVec,yVec);
        slabs{end+1} = s;	
    elseif (strcmp(slabInfo{k}.type,'Air') && useAir)
        if (slabInfo{k}.wx > 1e-9 && slabInfo{k}.wy > 1e-9)
            if useSym(3)
                s = block3(slabInfo{k}.wx,slabInfo{k}.wy,P.air,'base','center','pos',[slabInfo{k}.x slabInfo{k}.y P.air/2]);
            else
                s = block3(slabInfo{k}.wx,slabInfo{k}.wy,2*P.air,'base','center','pos',[slabInfo{k}.x slabInfo{k}.y 0]);
            end
            airSlab = s;
        end
    end
end
slabs = geomdel(geomcomp(slabs));

%% Draw holes
holes = {};
for k = 1:length(holeInfo)
	if (strcmp(holeInfo{k}.type,'Circle'))
        if (holeInfo{k}.r > 1e-9)
			h = circ2(holeInfo{k}.r,'pos',[holeInfo{k}.x holeInfo{k}.y]);
            holes{end+1} = h;
        end
    elseif (strcmp(holeInfo{k}.type,'Polygon'))
        xVec = [];
        yVec = [];
        for vert = 1:length(holeInfo{k}.x)
            xVec(end+1) = holeInfo{k}.x(vert);
            yVec(end+1) = holeInfo{k}.y(vert);
        end
        s = line2(xVec,yVec);
        holes{end+1} = s;	
    elseif (strcmp(holeInfo{k}.type,'Ellipse'))
		if (holeInfo{k}.rx > 1e-9 && holeInfo{k}.ry > 1e-9)
			h = ellip2(holeInfo{k}.rx,holeInfo{k}.ry,'base','center','pos',[holeInfo{k}.x holeInfo{k}.y]);
            holes{end+1} = h;
		end
	elseif (strcmp(holeInfo{k}.type,'Cross'))
		if (holeInfo{k}.h > 1e-9 && holeInfo{k}.w > 1e-9)
			h =cross(holeInfo{k}.h,holeInfo{k}.w,[holeInfo{k}.x holeInfo{k}.y]);
            holes{end+1} = h;
        end
	elseif (strcmp(holeInfo{k}.type,'Rectangle'))
		if (holeInfo{k}.wx > 1e-9 && holeInfo{k}.wy > 1e-9)
			h = rect2(holeInfo{k}.wx,holeInfo{k}.wy,'base','center','pos',[holeInfo{k}.x holeInfo{k}.y]);
            holes{end+1} = h;
        end	
    elseif (strcmp(holeInfo{k}.type,'Snowflake') || strcmp(holeInfo{k}.type,'Astrix'))
		if (holeInfo{k}.w > 1e-9 && holeInfo{k}.r > 1e-9)
			h = snowflake(holeInfo{k}.w,holeInfo{k}.r,[holeInfo{k}.x holeInfo{k}.y]);
            holes{end+1} = h;
        end
    end
end
holes = geomcomp(holes);
holes = fillet(holes,'radii',P.fillet);

%% Define and extrude unit cell (add air box if needed)
unitCell = slabs-holes;
if useSym(3)
	unitCell = extrude(unitCell,'distance',thi/2);
else
	unitCell = move(extrude(unitCell,'distance',thi),0,0,-thi/2);
end

geom = unitCell;
if ~isempty(airSlab)
    geom = geom+airSlab;
end

figure;
geomplot(geom);

clear s
s.objs = {geom};
fem.draw.s = s;

end

%% Functions to draw non-trivial hole shapes
function snowflake = snowflake(w,r,pos)
    global CORNER_FILLET_RADIUS;
    u1=fillet(rect2(w,r*2,'base','center','rot','30'),'radii',w/2);
    u2=fillet(rect2(w,r*2,'base','center','pos',[0,0],'rot','90'),'radii',w/2);
    u3=fillet(rect2(w,r*2,'base','center','pos',[0,0],'rot','150'),'radii',w/2);
%     u1=rect2(w,r*2,'base','center','rot','30');
%     u2=rect2(w,r*2,'base','center','pos',[0,0],'rot','90');
%     u3=rect2(w,r*2,'base','center','pos',[0,0],'rot','150');
    snowflake = geomcomp({u1,u2,u3});
    snowflake = fillet(geomdel(snowflake),'radii',CORNER_FILLET_RADIUS);
%     snowflake = geomdel(snowflake);
    snowflake = move(snowflake,pos);
end

function cross = cross(h,w,pos)
    global CORNER_FILLET_RADIUS;
    u1 = rect2(h,w,'base','center','pos',pos);
    u2 = rect2(w,h,'base','center','pos',pos);
    cross = geomcomp({u1,u2});
    cross = fillet(geomdel(cross),'radii',CORNER_FILLET_RADIUS);
end
