% Generates slab and hole information for a single nanobeam.
% The input structure P is expected to have the following fields:
%
% P.a = nominal lattice spacing; P.hh/hw = nominal hole height/width;
% P.w = beam width; P.mDef = maximum defect percentage;
% P.oblong = factor which controls ellipticity in the defect region;
% P.nDef = # of holes in 1/2 defect; P.nNom = # of nominal holes in 1/2 beam;
% P.useSym = 3 element vector detailing whether or not to use symmetry;
% P.stdDev = 2-element vector of standard deviation of hh and hw;
% P.stdDevPos = standard deviation of hole positions
%
% Sean Meenehan, 4/9/2014
%
% P.PMLLen = length of perfectly matching layer;
% Hengyun (Harry) Zhou, 7/11/15

function [slabs,holes,fixedBounds,startPML] = generateSlabHoleNanobeam(P)

a = P.a;
hh = P.hh;
hw = P.hw;
w = P.w;
mDef = P.mDef;
oblong = P.oblong;
nDef = P.nDef;
nNom = P.nNom;
useSym = P.useSym;
stdDev = P.stdDev;
stdDevPos = P.stdDevPos;
if isfield(P,'PMLLen')
    PMLLen = P.PMLLen;
else
    PMLLen = 2e-6;
end

    

% Set up positions and hole sizes for nanobeam
fs = [1-mDef*(1-((0:nDef-1)/nDef).^2),ones(1,nNom)];
if ~useSym(1)
    fs = [fliplr(fs(2:end)),fs];
end
fs = smooth(fs,1)';
as = a*(fs(1:end-1)+fs(2:end))/2;
hhs = hh*fs.^(1-oblong);
hws = hw*fs.^(1+oblong);
x = [0 cumsum(as)];
y = zeros(1,length(x));
beamLen = max(x)+a;
if ~useSym(1)
    x = x-max(x)/2;
end

% Create the beam, and air slabs
slab.type = 'Rectangle';
slab.wx = beamLen;
if useSym(1)
    slab.x = beamLen/2;
else
    slab.x = 0;
end
if useSym(2)
    slab.wy = w/2;
    slab.y = w/4;
else
    slab.wy = w;
    slab.y = 0;
end
slabs = {slab};

slab.type = 'Air';
if useSym(2)
    slab.wy = w/2+2e-6;
    slab.y = w/4+1e-6;
else
    slab.wy = w+4e-6;
    slab.y = 0;
end
slabs{end+1} = slab;

% Create the external slabs
if isfield(P,'mechPML')
    slab.type = 'Rectangle';
    slab.wx = PMLLen;
    slab.x = beamLen/2+PMLLen/2;
    if useSym(1)
        slab.x = slab.x+beamLen/2;
    end
    if useSym(2)
        slab.wy = w/2;
        slab.y = w/4;
    else
        slab.wy = w;
        slab.y = 0;
    end
    slabs{end+1} = slab;
    if ~useSym(1)
        slab.x = -slab.x;
        slabs{end+1} = slab;
    end
end


holes = {};
hole.type = 'Ellipse';
for k = 1:length(hhs)
    [hole.x,hole.y] = randomizePos(x(k),y(k),stdDevPos);
    hole.rx = randomize(hhs(k)/2,stdDev(1));
    hole.ry = randomize(hws(k)/2,stdDev(2));
    holes{end+1} = hole;
end

if isfield(P,'mechPML')
    if useSym(1)
        fixedBounds = {[beamLen+PMLLen,0,0],[1,0,0]};
    else
        fixedBounds = {[beamLen/2+PMLLen,0,0],[1,0,0],[-beamLen/2-PMLLen,0,0],[1,0,0]};
    end
else
    if useSym(1)
        fixedBounds = {[beamLen,0,0],[1,0,0]};
    else
        fixedBounds = {[beamLen/2,0,0],[1,0,0],[-beamLen/2,0,0],[1,0,0]};
    end
end

% PML starting position, no PML in y-direction
if isfield(P,'mechPML')
    if useSym(1)
        startPML = [beamLen,Inf];
    else
        startPML = [beamLen/2,Inf];
    end
else
    startPML = [Inf,Inf];
end


HHS=hhs'/a
HWS=hws'/a

end
