% Generates slab and hole information for a single nanobeam surrounded by a 
% cross structure.
% The input structure P is expected to have the following fields:
%
% P.a = nominal lattice spacing; P.hh/hw = nominal hole height/width;
% P.w = beam width; P.mDef = maximum defect percentage;
% P.oblong = factor which controls ellipticity in the defect region;
% P.nDef = # of holes in 1/2 defect; P.nNom = # of nominal holes in 1/2 beam;
% P.useSym = 3 element vector detailing whether or not to use symmetry;
% P.stdDev = 4-element vector of standard deviation of hh, hw, hC and nC;
% P.stdDevPos = standard deviation of hole position;
% P.aC = lattice spacing for cross; P.h/wC = cross height/width;
% P.nC = # of cross periods;
%
%
% Sean Meenehan, 3/5/14
%
% P.PMLLen = length of perfectly matching layer;
% P.PMLStart = starting radius or distance of PML;
% P.PMLShape = shape of PML, 'circ' or 'rect' supported;
% Hengyun (Harry) Zhou, 7/11/15

function [slabs,holes,fixedBounds,decayFunc] = generateSlabHoleNanobeamCross(P)

a = P.a;
hh = P.hh;
hw = P.hw;
w = P.w;
mDef = P.mDef;
oblong = P.oblong;
nDef = P.nDef;
nNom = P.nNom;
useSym = P.useSym;
stdDev = P.stdDev;
stdDevPos = P.stdDevPos;
aC = P.aC;
hC = P.hC;
wC = P.wC;
nC = P.nC;
if isfield(P,'PMLLen')
    PMLLen = P.PMLLen;
else
    PMLLen = 3e-6;
end
if isfield(P,'PMLShape')
    PMLShape = P.PMLShape;
else
    PMLShape = 'Rect';
end
if isfield(P,'PMLStart')
    PMLStart = max(P.PMLStart+aC*(nC+1),1.5*aC*(nC+2));
else
    PMLStart = 1.5*aC*(nC+2);
end

% Set up positions and hole sizes for nanobeam
fs = [1-mDef*(1-((0:nDef-1)/nDef).^2),ones(1,nNom)];
if ~useSym(1)
    fs = [fliplr(fs(2:end)),fs];
end
as = a*(fs(1:end-1)+fs(2:end))/2;
hhs = hh*fs.^(1-oblong);
hws = hw*fs.^(1+oblong);
x = [0 cumsum(as)];
y = zeros(1,length(x));
beamLen = max(x)+a;
if ~useSym(1)
    x = x-max(x)/2;
end

% Create the beam, and air slabs
slab.type = 'Rectangle';
slab.wx = beamLen;
if useSym(1)
    slab.x = beamLen/2;
else
    slab.x = 0;
end
if useSym(2)
    slab.wy = w/2;
    slab.y = w/4;
else
    slab.wy = w;
    slab.y = 0;
end
slabs = {slab};

slab.type = 'Air';
if useSym(2)
    slab.wy = w/2+2e-6;
    slab.y = w/4+1e-6;
else
    slab.wy = w+4e-6;
    slab.y = 0;
end

% Create the external slabs
if strcmp(PMLShape,'Circ')
    slab.type = 'SemiCircle';
    slab.r = PMLStart + PMLLen;
    slab.x = beamLen/2;
    if useSym(1)
        slab.x = slab.x+beamLen/2;
    end
    slab.y = 0;
    if useSym(2)
        slab.angstart = 0;
        slab.ang = pi/2;
    else
        slab.angstart = -pi/2;
        slab.ang = pi;
    end
    slabs{end+1} = slab;
    if ~useSym(1)
        slab.x = -slab.x;
        if useSym(2)
            slab.angstart = pi/2;
            slab.ang = pi/2;
        else
            slab.angstart = pi/2;
            slab.ang = pi;
        end
        slabs{end+1} = slab;
    end
    if useSym(1)
        decayFunc = ['(1./(1+1i*',num2str(P.mechPMLStr),'*(x>',num2str(beamLen),').*(((x-(',num2str(slab.x),...
            ')).^2+(y-(',num2str(slab.y),')).^2)>',num2str(PMLStart),'.^2)'...
            '.*((sqrt((x-(',num2str(slab.x),')).^2+(y-(',num2str(slab.y),')).^2)-',...
            num2str(PMLStart),')./',num2str(P.a),').^2)).^2'];
    else
        decayFunc = ['(1./(1+1i*',num2str(P.mechPMLStr),'*((x>',num2str(beamLen/2),').*(((x-(',...
            num2str(beamLen/2),')).^2+(y-(0)).^2)>',num2str(PMLStart),'.^2)'...
            '.*((sqrt((x-(',num2str(beamLen/2),')).^2+(y-(0)).^2)-',...
            num2str(PMLStart),')./',num2str(P.a),').^2',...
            '+(x<',num2str(-beamLen/2),').*(((x-(',...
            num2str(-beamLen/2),')).^2+(y-(0)).^2)>',num2str(PMLStart),'.^2)'...
            '.*((sqrt((x-(',num2str(-beamLen/2),')).^2+(y-(0)).^2)-',...
            num2str(PMLStart),')./',num2str(P.a),').^2',...
            '))).^2'];
        % PML starts from a circle at radius PMLStart and ends at radius
        % PMLStart+PMLLen
    end
else
    slab.type = 'Rectangle';
    slab.wx = PMLStart+PMLLen;
    slab.x = beamLen/2+(PMLStart+PMLLen)/2;
    if useSym(1)
        slab.x = slab.x+beamLen/2;
    end
    if useSym(2)
        slab.wy = PMLStart+PMLLen;
        slab.y = (PMLStart+PMLLen)/2;
    else
        slab.wy = 2*(PMLStart+PMLLen);
        slab.y = 0;
    end
    slabs{end+1} = slab;
    if ~useSym(1)
        slab.x = -slab.x;
        slabs{end+1} = slab;
    end
    if ~useSym(1)
        decayFunc = ['(1./(1+1i*',num2str(P.mechPMLStr),'*((abs(x)>',num2str(PMLStart+beamLen/2),').*', ...
                 '((abs(x)-',num2str(PMLStart+beamLen/2),')/',num2str(P.a),').^2',...
                 '+(abs(y)>',num2str(PMLStart),').*'...
                 '((abs(y)-',num2str(PMLStart),')/',num2str(P.a),').^2))).^2'];
    else
        decayFunc = ['(1./(1+1i*',num2str(P.mechPMLStr),'*((abs(x)>',num2str(PMLStart+beamLen),').*', ...
                 '((abs(x)-',num2str(PMLStart+beamLen),')/',num2str(P.a),').^2',...
                 '+(abs(y)>',num2str(PMLStart),').*'...
                 '((abs(y)-',num2str(PMLStart),')/',num2str(P.a),').^2))).^2'];
    end
end


% Add elliptical holes
holes = {};
hole.type = 'Ellipse';
for k = 1:length(hhs)
    [hole.x,hole.y] = randomizePos(x(k),y(k),stdDevPos);
    hole.rx = randomize(hhs(k)/2,stdDev(1));
    hole.ry = randomize(hws(k)/2,stdDev(2));
    holes{end+1} = hole;
end

% Add cross holes (no disorder implemented for now)
clear hole;
hole.type = 'Cross';
xH = beamLen/2;
if useSym(1)
    xH = beamLen;
end
yH = 0;
for xInd = 1:nC+1
    for yInd = 1:nC+1
        if ~(xInd==1 && yInd ==1)
            [hole.x,hole.y] = randomizePos(xH,yH,stdDevPos);
            hole.h = randomize(hC,stdDev(3));
            hole.w = randomize(wC,stdDev(4));
            holes{end+1} = hole;
            if ~useSym(1) && ~useSym(2)
                [hole.x,hole.y] = randomizePos(xH,-yH,stdDevPos);
                hole.h = randomize(hC,stdDev(3));
                hole.w = randomize(wC,stdDev(4));
                holes{end+1} = hole;
                [hole.x,hole.y] = randomizePos(-xH,-yH,stdDevPos);
                hole.h = randomize(hC,stdDev(3));
                hole.w = randomize(wC,stdDev(4));
                holes{end+1} = hole;
                [hole.x,hole.y] = randomizePos(-xH,yH,stdDevPos);
                hole.h = randomize(hC,stdDev(3));
                hole.w = randomize(wC,stdDev(4));
                holes{end+1} = hole;
            elseif ~useSym(1)
                [hole.x,hole.y] = randomizePos(-xH,yH,stdDevPos);
                hole.h = randomize(hC,stdDev(3));
                hole.w = randomize(wC,stdDev(4));
                holes{end+1} = hole;
            elseif ~useSym(2)
                [hole.x,hole.y] = randomizePos(xH,-yH,stdDevPos);
                hole.h = randomize(hC,stdDev(3));
                hole.w = randomize(wC,stdDev(4));
                holes{end+1} = hole;
            end 
        end
        yH = yH+aC;
    end
    xH = xH+aC;
    yH = 0;
end


if useSym(1)
    fixedBounds = {[beamLen+PMLStart+PMLLen,0,0],[1,0,0]};
else   
    fixedBounds = {[beamLen/2+PMLStart+PMLLen,0,0],[1,0,0],[-beamLen/2-PMLStart-PMLLen,0,0],[1,0,0]};
end
fixedBounds{end+1} = [0,PMLStart+PMLLen,0];
fixedBounds{end+1} = [0,1,0];
fixedBounds{end+1} = [0,-PMLStart-PMLLen,0];
fixedBounds{end+1} = [0,1,0];

if useSym(1)
    startPML = [beamLen+aC*nC,aC*nC];
else
    startPML = [beamLen/2+aC*nC,aC*nC];
end