% This is a high level script that takes a certain geometry and runs band
% structure calculations as well as calculations of the nanobeam frequency 
% and quality factor
% Inputs:
%   fn: filename prefix
%   P: struct containing information about system
%   R: struct containing parameters for simulation
% Outputs:
%   dsm: mechanical simulation results of cavity
%   dsn: band structure simulations of nanobeam holes
%   dsc: band structure simulations of phononic shield
%   dso: optical simulation results of cavity
% Hengyun (Harry) Zhou, 08/19/2015

function [restab,dsm,dso] = runFullpG_noBG(P,R,fn)

%close all;
warning('off','all')

%% Run code for geometry extraction and mechanical simulation
%
[slabInfo,holeInfo,fixedBounds,startPML] = generateSlabHoleNanobeamHalfCrossSubdom(P);
P.pG.slabInfo = slabInfo;
P.pG.holeInfo = holeInfo;
P.pG.fixedBounds = fixedBounds;
P.pG.startPML = startPML;
%}
dsm = runMechSubdom(P,'Q',R);
% find 5 highest Q mechanical modes
Qmvals = dsm.freqs./dsm.decay;
[~,sortInd] = sort(Qmvals(:),'descend');
mModes = sortInd(1:5);
%}

%% Run code for optical simulation
%
Po = P;
Po.type = 'Nanobeam';
Po.useAir = 1;
dso = runOptical(Po);
%}

%% Run code for optomechanical coupling extraction
%
p11 = -0.101;
p12 = 0.0094;
p44 = -0.051;   % temporarily taking numbers from JCB's files
ds2 = calculateGpeCubic(dso.fem,dsm.fem,mModes,P.useSym,p11,p12,p44);
%}

%% Save relevant results for best optomechanical coupling case
res.mechFreq = ds2.mechFreq;
res.optWvl = ds2.optWvl;
res.gBndMax = ds2.gBndMax;
res.gMax = ds2.gMax;
res.gStrMax = ds2.gStrMax;
res.oSol = ds2.oSol;
res.mSol = ds2.mSol;
res.Qo = ds2.Q;
res.Qm = dsm.freqs(ds2.mSol)/dsm.decay(ds2.mSol);
res.stdDev = P.stdDev;
res.stdDevPos = P.stdDevPos;
res.nC = P.nC;
oSol = ds2.oSol
mSol = ds2.mSol
mechFreq = res.mechFreq
Qm = res.Qm
%save(strcat(fn,'_res.mat'),'res','restab')

% %% Run code for hole averaging and nanobeam mechanical band structure
% %
% Pn = rmfield(P,'pG');
% Pn.neigs = 10;
% Pn.kPts = 20;
% Pn.type = 'Nanobeam';
% Pn.freq = 0;    % start band structure calculation from zero frequency
% Pn.useSym = [0 0 0];   
% % Pn stores info for the nanobeam band structure calculation
% dsnfull = runMechBands(Pn);
% fnfull = dsnfull.F;
% Pn.neigs = 5;
% Pn.useSym = [0 1 1];  
% dsnsym = runMechBands(Pn);
% fnsym = dsnsym.F;
% clear dsnfull
% clear dsnsym
% %}
% 
% %% Run code for cross averaging and mechanical band structure
% %
% Pc = rmfield(P,'pG');
% Pc.type = 'Cross';
% Pc.freq = 0;
% Pc.kPts = 20;
% Pc.neigs = 10;
% Pc.a = P.aC;
% Pc.h = P.hC;
% Pc.w = P.wC;
% Pc.useSym = [0 0 0];
% dsc = runMechBands(Pc);
% fc = dsc.F;
% clear dsc
% %}
restab = [P.stdDev,P.stdDevPos,P.nC,res.mechFreq,res.optWvl,res.gMax,res.gBndMax,res.gStrMax,res.oSol,res.mSol,res.Qo,res.Qm];
%% Save results
%{
save(strcat(fn,'_res.mat'),'res','fnfull','fnsym','fc');
%}
warning('on','all')


