% This function runs a simulation for the optical modes of a photonic crystal cavity.
% The input structure P is assumed to have the following entries (plus additional
% geometry parameters, see createCavity.m):
%
% P.neigs = # of eigenvalues to find; P.lambda = target wavelength;
% P.useSym/sym = which planes have symmetry, and what type of symmetry;
% P.n = index of refraction of the material; 
%
% varargin = additional arguments, submitted as 'variableName',value pairs;
%            supported arguments are:
%               maxDoF = maximum # of degrees of freedom for the simulation;
%               lambdaDiv = wavelength division factor for max. mesh size
%               meshQuality = starting 'hauto' param for mesh quality
%               QThresh = threshold optical Q for "real" cavities
%
% Sean Meenehan, 1/7/15

function ds = runOptical(P,varargin)

flclear fem

c = 299792458;

useSym = P.useSym;
sym = P.sym;
neigs = P.neigs;
n = P.n;
lambda = P.lambda;

% Parse varargin
maxDoF = 3e6;
lambdaDiv = 5;
meshQuality = 5;
QThresh = 1e4;
if(mod(length(varargin),2)>0); error('Incorrect parameter specification.'); end;
for k=1:2:length(varargin)
    if (exist(varargin{k},'var'))
        eval([varargin{k},' = ',num2str(varargin{k+1}),';']);
    else 
        display([varargin{k} ' is not a valid parameter.']);
    end
end

%% Set up geometry
clear draw
draw.s.objs = {};
draw.s.name = {};
draw.s.tags = {};
fem.draw = draw;
[fem,P,~] = createCavity(fem,P,'useAir',1);
fem.geom = geomcsg(fem);
ds.P = P;

%% Set up application mode
clear appl
appl.mode.class = 'ElectromagneticWaves';
appl.module = 'RF';
appl.gporder = 4;
appl.cporder = 2;
appl.assignsuffix = '_rfw';
clear prop
prop.analysis='eigen';
appl.prop = prop;

%% Set up boundary conditions
clear bnd
bnd.type = {'E0','H0','SC','cont'};
nfaces = geominfo(fem.geom,'out','no','Od',2); % # of faces
adj = geominfo(fem.geom,'out','adj','odp',[2;3]); % boundary/subdomain adjacency info

%Starting all boundaries as continuity
bnd.ind = 4*ones(1,nfaces);

% Set all exterior boundaries as scattering; recall subdomain 1 is always
% the complete simulation volume
bnd.ind(find(adj{1}(:,1))) = 3;

% For any symmetries used, overwrite the scattering boundary condition as
% needed
bndE0 = [];
bndH0 = [];
if useSym(1)
    aux = bndindex(fem.geom,[0 0 0],[1 0 0]);
    if sym(1)
        bndH0(end+1:end+length(aux)) = aux; clear aux
    else
        bndE0(end+1:end+length(aux)) = aux; clear aux
    end
end
if useSym(2)
    aux = bndindex(fem.geom,[0 0 0],[0 1 0]);
    if sym(2)
        bndH0(end+1:end+length(aux)) = aux; clear aux
    else
        bndE0(end+1:end+length(aux)) = aux; clear aux
    end
end
if useSym(3)
    aux = bndindex(fem.geom,[0 0 0],[0 0 1]);
    if sym(3)
        bndH0(end+1:end+length(aux)) = aux; clear aux
    else
        bndE0(end+1:end+length(aux)) = aux; clear aux
    end
end
bnd.ind(bndE0) = 1;
bnd.ind(bndH0) = 2;

appl.bnd = bnd;

%% Set up material properties
nobjs = geominfo(fem.geom,'out','no','Od',3); %number of solid objects
clear equ
equ.epsilonr = {1,n^2};
equ.n = {1,n};
equ.matparams = {'epsr','n'};
equ.ind = 1*ones(1,nobjs);

% Find the air, all other subdomains should be the cavity material
bndTop = bndindex(fem.geom,[0,0,P.air],[0,0,1]);
indCav = 1:nobjs;
indAir = find(adj{1}(bndTop,:));
indAir(indAir==1) = [];
indAir = indAir-1;
indCav(indCav==indAir) = [];
equ.ind(indCav) = 2;
ds.indCav = indCav;
appl.equ = equ;

fem.appl{1} = appl;
fem.border = 1;
fem=multiphysics(fem);
%
%% Solve problem with the best mesh possible (starting from meshQuality)
ok=0;
display(['Meshing with quality: ',num2str(meshQuality)]);
fem.mesh=meshinit(fem, ...
    'hauto',meshQuality, ...
	'hmax',lambda/lambdaDiv, ...
	'hmaxsub',[indCav; lambda/(n*lambdaDiv)*ones(1,length(indCav))]);
fem.xmesh=meshextend(fem);
dofs = xmesh_dofs(fem);
display(['Estimated # of DoFs: ',num2str(dofs)]);
if dofs>=maxDoF
    error('Too large!');
end
while ok==0 && meshQuality<10
    while dofs>=maxDoF && meshQuality<9
        meshQuality = meshQuality+1;
        display(['Meshing with quality: ',num2str(meshQuality)]);
        fem.mesh=meshinit(fem, ...
            'hauto',meshQuality, ...
            'hmax',lambda/lambdaDiv, ...
            'hmaxsub',[indCav; lambda/(n*lambdaDiv)*ones(1,length(indCav))]);
        fem.xmesh=meshextend(fem);
        dofs = xmesh_dofs(fem);
        display(['Estimated # of DoFs: ',num2str(dofs)]);
    end
    pause(2);
    try
        display('Solving');
        fem.sol=femeig(fem, ...
               'solcomp',{'tExEyEz10','tExEyEz20','tExEyEz21'}, ...
               'outcomp',{'tExEyEz10','tExEyEz20','tExEyEz21'}, ...
               'neigs',neigs, ...
               'shift',0-1i*2*pi*c/lambda, ...
               'symmetric','on', ...
               'linsolver','pardiso');
        ok = 1;
        display('Done')
    catch 
        display(lasterr)
        meshQuality = meshQuality+1;        
        display(['Decreasing mesh quality. New Mesh = ', num2str(meshQuality)]);
        fem.mesh=meshinit(fem, ...
            'hauto',meshQuality, ...
            'hmax',lambda/lambdaDiv, ...
            'hmaxsub',[indCav; lambda/(n*lambdaDiv)*ones(1,length(indCav))]);
        fem.xmesh=meshextend(fem);
        dofs = xmesh_dofs(fem);
        display(['Estimated # of DoFs: ',num2str(dofs)]);
    end
end           
     
%% Post-processing
ds.lambdaAll = c./((fem.sol.lambda)/(2*pi));
ds.QAll = imag(ds.lambdaAll)./(2*real(ds.lambdaAll));
ds.realSol = find(ds.QAll>QThresh);
ds.lambda = imag(ds.lambdaAll(ds.realSol));
ds.Q = ds.QAll(ds.realSol);
Q=ds.Q
%}
ds.fem = fem;