% This function calculates the optomechanical couplings for the mechanical
% modes stored in mfem (with solution indices given by mModes), with all 
% high-Q optical resonances in ofem. The output ds.g contains the value of 
% g0 in Hz. The optical effective volumes (in units of cubic half-wavelengths) 
% and mechanical zero-point amplitudes (in units of m) are also stored in 
% ds.Veff and ds.xzpf, respectively. The maximum g, and corresponding 
% optical and mechanical mode indices are also stored.
% 
% In addition, we include photoelastic effects in the
% calculation. We assume a cubic structure (e.g. Silicon), so that the
% photoelastic tensor has only three relevant independent elements: two
% hydrostatic terms (p11 and p12) and a shear stress term (p44)
%
% Sean Meenehan, 12/4/11

function ds = calculateGpeCubic_AllL(ofem,mfem,mModes,sym,p11,p12,p44,varargin)

global oInt
global oIntCnt
global oIntFem
global oIntEx
global oIntExCnt
global oIntEy
global oIntEyCnt
global oIntEz
global oIntEzCnt
global oIntExy
global oIntExyCnt
global oIntExz
global oIntExzCnt
global oIntEyz
global oIntEyzCnt
global oSol
global xLim 
global yLim

xLim = 1e-3;
yLim = 1e-3;

c = 299792458; % m/s
hbar = 1.05457148e-34; % J*s

oIntFem = ofem;
n = ofem.appl{1}.equ.n{end};
rho = mfem.appl{1}.equ.rho{end};


% bnds = 1:length(mfem.bnd.ind);
% bndsNil = [];
% aux = bndindex(mfem.geom,[0,0,0],[1,0,0]);
% bndsNil(end+1:end+length(aux)) = aux;
% aux = bndindex(mfem.geom,[0,0,0],[0,1,0]);
% bndsNil(end+1:end+length(aux)) = aux;
% aux = bndindex(mfem.geom,[0,0,0],[0,0,1]);
% bndsNil(end+1:end+length(aux)) = aux;   % I think this is to eliminate symmetry plane boundaries
% bnds = setdiff(bnds,bndsNil);

bnds = 1:length(mfem.bnd.ind);
bndsNil = [];
if sym(1)
	aux = bndindex(mfem.geom,[0,0,0],[1,0,0]);
	bndsNil(end+1:end+length(aux)) = aux;
end
if sym(2)
	aux = bndindex(mfem.geom,[0,0,0],[0,1,0]);
	bndsNil(end+1:end+length(aux)) = aux;
end
if sym(3)
	aux = bndindex(mfem.geom,[0,0,0],[0,0,1]);
	bndsNil(end+1:end+length(aux)) = aux;
end
bnds = setdiff(bnds,bndsNil);


ds.gMax = 0;

% Locate high-Q modes
fAll = abs(imag(ofem.sol.lambda))/2/pi;
QAll = abs(imag(ofem.sol.lambda)./(2*real(ofem.sol.lambda)));
realSol = find(QAll>1e5);
fOs = fAll(realSol);
lambdas = c./fOs;
Qs = QAll(realSol);
symFac = 2^sum(sym);

if length(varargin)==2
    goodWvls = find(lambdas>varargin{1}&lambdas<varargin{2});
    realSol = realSol(goodWvls);
    fOs = fOs(goodWvls);
    lambdas = lambdas(goodWvls);
    Qs = Qs(goodWvls);
end

% Loop over real (i.e. high Q) optical modes
for j = 1:length(realSol)
    display(['Processing optical mode ',num2str(j),'...']);
    oSol = realSol(j);
    
    % Calculating optical normalizations and effective volumes
    LV(j) = postint(ofem,'normE_rfw^2*epsilon_rfw','solnum',oSol,'Edim',3);
    LVmax = postmax(ofem,'normE_rfw^2*epsilon_rfw','solnum',oSol,'Edim',3);
    
    ds.Veff(j) = (symFac*LV(j)/LVmax)/(lambdas(j)/(2*n))^3;
    
    % prep the boundary integral
    oInt = {};
    oIntCnt = 1;
    display('Prepping boundaries...');
    postint(mfem,'oIntBndPrep(x,y,z)','dl',bnds,'edim',2);
    
    % Prep the volume integral
    [oIntEx,oIntEy,oIntEz,oIntExy,oIntExz,oIntEyz] = deal({},{},{},{},{},{});
    [oIntExCnt,oIntEyCnt,oIntEzCnt,oIntExyCnt,oIntExzCnt,oIntEyzCnt] = deal(1,1,1,1,1,1);
    display('Prepping volume...');
    postint(mfem,'oIntVolPrep(x,y,z)','edim',3);
    
    % Loop over all mechanical modes
    for k = 1:length(mModes)
        mSol = mModes(k);
        wM = abs(imag(mfem.sol.lambda(mSol)));
        display(['Mechanical mode ',num2str(k),'...']);
        
        oIntCnt = 1;
	    L = postint(mfem,'oIntRun(x,y,z,''oInt'')*(u*nx+v*ny+w*nz)','dl',bnds,'edim',2,'solnum',mSol);
        
        % Caclulate the contribution due to photoelasticity 
        [oIntExCnt,oIntEyCnt,oIntEzCnt,oIntExyCnt,oIntExzCnt,oIntEyzCnt] = deal(1,1,1,1,1,1);
        LDiv = postint(mfem,['oIntRun(x,y,z,''oIntEx'')*(ex_smsld*',num2str(p11),'+(ey_smsld+ez_smsld)*',num2str(p12),')+', ...
                             'oIntRun(x,y,z,''oIntEy'')*(ey_smsld*',num2str(p11),'+(ex_smsld+ez_smsld)*',num2str(p12),')+', ...
                             'oIntRun(x,y,z,''oIntEz'')*(ez_smsld*',num2str(p11),'+(ex_smsld+ey_smsld)*',num2str(p12),')'], ...
                             'edim',3,'solnum',mSol);

        LShear = postint(mfem,[num2str(p44),'*(2*exy_smsld*oIntRun(x,y,z,''oIntExy'')', ...
                               '+2*exz_smsld*oIntRun(x,y,z,''oIntExz'')', ...
                               '+2*eyz_smsld*oIntRun(x,y,z,''oIntEyz''))'], ...
                               'edim',3,'solnum',mSol);
        [oIntExCnt,oIntEyCnt,oIntEzCnt,oIntExyCnt,oIntExzCnt,oIntEyzCnt] = deal(1,1,1,1,1,1);                   
        LP11EySyy = postint(mfem,['oIntRun(x,y,z,''oIntEy'')*(ey_smsld*',num2str(p11),')'], ...
                        'edim',3,'solnum',mSol);
   
        % Get the maximum mechanical displacement (for proper
        % normalization)
        maxDisp = postmax(mfem,'abs(disp_smsld)','solnum',mSol);
        
        % Calculate the zero-point amplitude if we're on the first loop
        if j == 1
            % Calculate effective mass
            dispSq = postint(mfem,'disp_smsld^2','solnum',mSol);
            dispSqMax = postmax(mfem,'disp_smsld^2','solnum',mSol);
            meff = symFac*rho*(dispSq/dispSqMax);

            % zero-point amplitude
            ds.xzpf(k) = sqrt(hbar/(2*meff*wM));
        end
        ds.gBnd(j,k) = -ds.xzpf(k)*fOs(j)*(L/LV(j)/2/maxDisp); % g due to moving boundaries in Hz
        ds.gStr(j,k) = -ds.xzpf(k)*fOs(j)*(-n^4*(LDiv+LShear)/LV(j)/2/maxDisp); % g due to strain in Hz 
        ds.g(j,k) = ds.gBnd(j,k)+ds.gStr(j,k); % g in Hz
        if abs(ds.g(j,k)) > abs(ds.gMax)
            ds.gBndMax = ds.gBnd(j,k);
            ds.gStrMax = ds.gStr(j,k);
            ds.LDiv = LDiv;
            ds.LShear = LShear;
            ds.gMax = ds.g(j,k);
            ds.oSol = oSol;
            ds.mSol = mSol;
            ds.optWvl = lambdas(j);
            ds.mechFreq = wM/2/pi;
            ds.Q = Qs(j);
            % All L terms
            ds.LP11EySyy = LP11EySyy;
        end
    end
end

%clear global
