% This function handles the common geometry creation tasks, outsourcing
% the actual structure specific tasks to auxiliary functions to draw the
% slab/holes. Currently this function only supports single layer cavities.
% Inputs are:
%
% fem = FEM structure 
% P = structure containing geometry information specific to the cavity in
%     question; see associated generation files for information; fields
%     which are necessary for this function are:
%       P.type = string denoting the type of cavity to be drawn
%       P.useSym = 3 element vector denoting whether symmetry planes are used;
%       P.thi = slab thickness;
%       P.rFillet = fillet radius for snowflake corners (optional)
%       P.air = thickness of air slab (if used)
%
% varargin = additional arguments, submitted as 'variableName',value pairs;
%            supported arguments are:
%               useAir = whether or not to add an external air space
%                        (optical vs. mechanical sims)
%
% Sean Meenehan, 1/7/15
%
% Added fillet to crosses and half-cross geometry
% HZ, 07/13/15

function [fem,P,fixedBounds,startPML] = createCavitySubdom_tiltWall_roughBeam(fem,P,varargin)
% Parse varargin
global a;
useAir = 0;
scaleTopY = P.scaleTopY;
if(mod(length(varargin),2)>0); error('Incorrect parameter specification.'); end;
for k=1:2:length(varargin)
    if (exist(varargin{k},'var'))
        eval([varargin{k},' = ',num2str(varargin{k+1}),';']);
    else 
        display([varargin{k} ' is not a valid parameter.']);
    end
end

useSym = P.useSym;
thi = P.thi;

% globals
global CORNER_FILLET_RADIUS;
CORNER_FILLET_RADIUS = 0;
if isfield(P,'rFillet')
    CORNER_FILLET_RADIUS = P.rFillet;
end

%% Select cavity type. To add new cavity type, create an appropriate generate* function and add here
if isfield(P,'pG')
    % Information from patch geometry is generated using patchGeom. 
    slabInfo = P.pG.slabInfo;
    holeInfo = P.pG.holeInfo;
    fixedBounds = P.pG.fixedBounds;
    startPML = P.pG.startPML;
elseif strcmp(P.type,'Nanobeam')
    [slabInfo,holeInfo,fixedBounds,startPML] = generateSlabHoleNanobeam(P);
elseif strcmp(P.type,'NanobeamCross')
    [slabInfo,holeInfo,fixedBounds,startPML] = generateSlabHoleNanobeamCross(P);
elseif strcmp(P.type,'NanobeamHalfCross')
    [slabInfo,holeInfo,fixedBounds,startPML] = generateSlabHoleNanobeamHalfCrossSubdom(P);
elseif strcmp(P.type,'SideCplNanobeam')
	[slabInfo,holeInfo,fixedBounds,startPML] = generateSlabHoleSideCplNanobeam(P);
elseif strcmp(P.type,'SideCplNanobeamNoReflector')
	[slabInfo,holeInfo,fixedBounds,startPML] = generateSlabHoleSideCplNanobeamNoReflector(P);
elseif strcmp(P.type,'SnowflakeH1')
	[slabInfo,holeInfo,fixedBounds,startPML] = generateSlabHoleSnowflakeH1(P);
elseif strcmp(P.type,'SnowflakeL3')
	[slabInfo,holeInfo,fixedBounds,startPML] = generateSlabHoleSnowflakeL3(P);
elseif strcmp(P.type,'SnowflakeL3Mod')
	[slabInfo,holeInfo,fixedBounds,startPML] = generateSlabHoleSnowflakeL3Mod(P);
elseif strcmp(P.type,'SnowflakeRMod')
	[slabInfo,holeInfo,fixedBounds,startPML] = generateSlabHoleSnowflakeRMod(P);
elseif strcmp(P.type,'SnowflakeW1EllipseCenter')
    [slabInfo,holeInfo,fixedBounds,startPML] = generateSlabHoleSnowflakeW1EllipseCenter(P);
elseif strcmp(P.type,'CircleL3')
	[slabInfo,holeInfo,fixedBounds,startPML] = generateSlabHoleCircleL3(P);
elseif strcmp(P.type,'CircleL3Mod')
	[slabInfo,holeInfo,fixedBounds,startPML] = generateSlabHoleCircleL3Mod(P);
else
	error(['Unrecognized cavity type: ',P.type]);
end

%% Draw slabs
slabs = {};
slab1 = {};
airSlab = [];
PMLSlabs = {};
PMLSlabcut = {};
roughBeam = {};
for k = 1:length(slabInfo)
    if (strcmp(slabInfo{k}.type,'Rectangle'))
		if (slabInfo{k}.wx > 1e-9 && slabInfo{k}.wy > 1e-9)
			s = rect2(slabInfo{k}.wx,slabInfo{k}.wy,'base','center','pos',[slabInfo{k}.x slabInfo{k}.y]);
            if k==1
                slab1{end+1} = s;
            else
                slabs{end+1} = s;
            end
            
        end	
    elseif (strcmp(slabInfo{k}.type,'Taper'))
        if (slabInfo{k}.len > 1e-9 && slabInfo{k}.wL > 1e-9 && slabInfo{k}.wR > 1e-9)
			s = line2([slabInfo{k}.x-slabInfo{k}.len/2,slabInfo{k}.x+slabInfo{k}.len/2, ...
                       slabInfo{k}.x+slabInfo{k}.len/2,slabInfo{k}.x-slabInfo{k}.len/2], ...
                      [slabInfo{k}.y+slabInfo{k}.wL/2,slabInfo{k}.y+slabInfo{k}.wR/2, ...
                       slabInfo{k}.y-slabInfo{k}.wR/2,slabInfo{k}.y-slabInfo{k}.wL/2]);
            slabs{end+1} = s;
        end	
    elseif (strcmp(slabInfo{k}.type,'Air') && useAir)
        if (slabInfo{k}.wx > 1e-9 && slabInfo{k}.wy > 1e-9)
            if useSym(3)
                s = block3(slabInfo{k}.wx,slabInfo{k}.wy,P.air,'base','center','pos',[slabInfo{k}.x slabInfo{k}.y P.air/2]);
            else
                s = block3(slabInfo{k}.wx,slabInfo{k}.wy,2*P.air,'base','center','pos',[slabInfo{k}.x slabInfo{k}.y 0]);
            end
            airSlab = s;
        end
    elseif (strcmp(slabInfo{k}.type,'SemiCircle'))
		if (slabInfo{k}.r > 1e-9)
			s = arc2(slabInfo{k}.x,slabInfo{k}.y,slabInfo{k}.r,slabInfo{k}.r,slabInfo{k}.angstart,0,slabInfo{k}.ang);
            slabs{end+1} = s;
        end
    elseif (strcmp(slabInfo{k}.type,'PML'))
        if (slabInfo{k}.wx > 1e-9 && slabInfo{k}.wy > 1e-9)
			s = rect2(slabInfo{k}.wx,slabInfo{k}.wy,'base','center','pos',[slabInfo{k}.x slabInfo{k}.y]);
                PMLSlabs{end+1} = s;
        end
    elseif (strcmp(slabInfo{k}.type,'PMLcut'))
        if (slabInfo{k}.wx > 1e-9 && slabInfo{k}.wy > 1e-9)
			s = rect2(slabInfo{k}.wx,slabInfo{k}.wy,'base','center','pos',[slabInfo{k}.x slabInfo{k}.y]);
                PMLSlabcut{end+1} = s;
        end
    elseif (strcmp(slabInfo{k}.type,'roughBeam'))
        s = poly2(slabInfo{k}.x,slabInfo{k}.y);
                roughBeam{end+1} = s;
    end
end

%figure;
%geomplot(geomcomp(slabs));
%slabs = geomdel(geomcomp(slabs));
slab1 = geomcomp(slab1);
slabs = geomcomp(slabs);
PMLSlabs = geomdel(geomcomp(PMLSlabs));
roughBeam = geomcomp(roughBeam);
figure;
geomplot(roughBeam);
a = PMLSlabcut;
PMLSlabcut = geomdel(geomcomp(PMLSlabcut));
%% Draw holes
holes = {};
PMLholes = {};
for k = 1:length(holeInfo)
	if (strcmp(holeInfo{k}.type,'Circle'))
        if (holeInfo{k}.r > 1e-9)
			h = circ2(holeInfo{k}.r,'pos',[holeInfo{k}.x holeInfo{k}.y]);
            holes{end+1} = h;
        end
    elseif (strcmp(holeInfo{k}.type,'Ellipse'))
		if (holeInfo{k}.rx > 1e-9 && holeInfo{k}.ry > 1e-9)
			h = ellip2(holeInfo{k}.rx,holeInfo{k}.ry,'base','center','pos',[holeInfo{k}.x holeInfo{k}.y]);
            holes{end+1} = h;
		end
	elseif (strcmp(holeInfo{k}.type,'Cross'))
		if (holeInfo{k}.h1 > 1e-9 && holeInfo{k}.w1 > 1e-9)
			h =cross(holeInfo{k}.h1,holeInfo{k}.w1,holeInfo{k}.h2,holeInfo{k}.w2,[holeInfo{k}.x holeInfo{k}.y]);
            PMLholes{end+1} = h;
            %holes{end+1} = h;
        end
	elseif (strcmp(holeInfo{k}.type,'Rectangle'))
		if (holeInfo{k}.wx > 1e-9 && holeInfo{k}.wy > 1e-9)
			h = rect2(holeInfo{k}.wx,holeInfo{k}.wy,'base','center','pos',[holeInfo{k}.x holeInfo{k}.y]);
            holes{end+1} = h;
        end	
    elseif (strcmp(holeInfo{k}.type,'Snowflake') || strcmp(holeInfo{k}.type,'Astrix'))
		if (holeInfo{k}.w > 1e-9 && holeInfo{k}.r > 1e-9)
			h = snowflake(holeInfo{k}.w,holeInfo{k}.r,[holeInfo{k}.x holeInfo{k}.y]);
            holes{end+1} = h;
        end
    elseif (strcmp(holeInfo{k}.type,'Polygon')) % hole specified by boundary
        h = poly2(holeInfo{k}.x,holeInfo{k}.y);
        holes{end+1} = h;
    end
    
    
end
holes = geomcomp(holes);

if (P.nC ~=0)   
    PMLholes = geomcomp(PMLholes);
    PMLholes = fillet(PMLholes,'radii',P.crossfillet);
end

%% Define and extrude cavity (add air box if needed)
if P.roughBeamSections == 0
    cavity1 = slab1;
else
    cavity1 = roughBeam;
end
cavity2 = slabs;

if  P.tiltWall == 0 
    cavity1 = cavity1+cavity2-holes;
else
    cavity1 = cavity1+cavity2;
end
cavity1 = geomdel(cavity1);

if P.useSym(3)
	cavity1 = extrude(cavity1,'distance',thi/2,'scale',[1;scaleTopY]);
    %cavity2 = extrude(cavity2,'distance',thi/2);
    PMLSlabs = extrude(PMLSlabs,'distance',thi/2,'scale',[1;scaleTopY]);
    PMLSlabcut = extrude(PMLSlabcut,'distance',thi/2);
    holes = extrude(holes,'distance',thi/2,'scale',[1;1/scaleTopY]); % holes scale the reverse way
    if (P.nC ~=0)
    %PMLholes = extrude(PMLholes,'distance',thi/2,'scale',[1;scaleTopY]); %PMLholes cant be scaled
    PMLholes = extrude(PMLholes,'distance',thi/2,'scale',[1;1]);
    end
else
	cavity1 = move(extrude(cavity1,'distance',thi,'scale',[1;scaleTopY]),0,0,-thi/2);
    %cavity2 = move(extrude(cavity2,'distance',thi),0,0,-thi/2);
    PMLSlabs = move(extrude(PMLSlabs,'distance',thi,'scale',[1;scaleTopY]),0,0,-thi/2);
    PMLSlabcut = move(extrude(PMLSlabcut,'distance',thi),0,0,-thi/2);
    holes = move(extrude(holes,'distance',thi,'scale',[1;1/scaleTopY]),0,0,-thi/2);
    if (P.nC ~=0)
    %PMLholes = move(extrude(PMLholes,'distance',thi,'scale',[1;1/scaleTopY]),0,0,-thi/2);
    PMLholes = move(extrude(PMLholes,'distance',thi,'scale',[1;1]),0,0,-thi/2); %PMLholes cant be scaled
    end
end



if ~isempty(PMLSlabs)
    for k = 1:length(PMLSlabs)
        cavity1 = cavity1-PMLSlabs{k};
    end
end


PMLSlabs = PMLSlabs*PMLSlabcut;
PMLSlabs = geomdel(PMLSlabs);



% if (P.nC ~=0)
%     holes=holes+PMLholes;
% end






geom = cavity1;%+cavity2;
%geom = geomdel(geom);

if ~isempty(airSlab)
    geom = geom+airSlab;
end
if ~isempty(PMLSlabs)
    for k = 1:length(PMLSlabs)
        geom = geom+PMLSlabs{k};
    end
end

if  P.tiltWall == 0 
    
else
    geom = geom - holes;
end

if (P.nC ~=0)
    geom = geom - PMLholes;
end
    
figure;
geomplot(geom);

clear s
s.objs = {geom};
fem.draw.s = s;

end

%% Functions to draw non-trivial hole shapes
function snowflake = snowflake(w,r,pos)
    global CORNER_FILLET_RADIUS;
    u1=fillet(rect2(w,r*2,'base','center','rot','30'),'radii',w/2);
    u2=fillet(rect2(w,r*2,'base','center','pos',[0,0],'rot','90'),'radii',w/2);
    u3=fillet(rect2(w,r*2,'base','center','pos',[0,0],'rot','150'),'radii',w/2);
%     u1=rect2(w,r*2,'base','center','rot','30');
%     u2=rect2(w,r*2,'base','center','pos',[0,0],'rot','90');
%     u3=rect2(w,r*2,'base','center','pos',[0,0],'rot','150');
    snowflake = geomcomp({u1,u2,u3});
    snowflake = fillet(geomdel(snowflake),'radii',CORNER_FILLET_RADIUS);
%    snowflake = fillet(geomdel(snowflake),'edges',[1,4,8,12,9,5;3,2,6,11,10,7],'radii',CORNER_FILLET_RADIUS);
    snowflake = move(snowflake,pos);
end

function cross = cross(h1,w1,h2,w2,pos)
    global CORNER_FILLET_RADIUS;
    u1 = rect2(h1,w1,'base','center','pos',pos);
    u2 = rect2(w2,h2,'base','center','pos',pos);
    cross = geomcomp({u1,u2});
    cross = fillet(geomdel(cross),'radii',CORNER_FILLET_RADIUS);
end
