% Generates slab and hole information for a single nanobeam surrounded by a 
% cross structure.
% The input structure P is expected to have the following fields:
%
% P.a = nominal lattice spacing; P.hh/hw = nominal hole height/width;
% P.w = beam width; P.mDef = maximum defect percentage;
% P.oblong = factor which controls ellipticity in the defect region;
% P.nDef = # of holes in 1/2 defect; P.nNom = # of nominal holes in 1/2 beam;
% P.useSym = 3 element vector detailing whether or not to use symmetry;
% P.stdDev = 4-element vector of standard deviation of hh, hw, hC and wC;
% P.stdDevPos = standard deviation of hole position;
% P.aC = lattice spacing for cross; P.h/wC = cross height/width;
% P.nC = # of cross periods;
%
% Sean Meenehan, 3/5/14
%
% P.PMLLen = length of perfectly matching layer;
% P.wS = width of supporting region where crosses are;
% Note that useSym(2) is not supported because there is no symmetry in this
% direction
% P.xC, P.yC = location of innermost center of cross relative to end of
% nanobeam
% Hengyun (Harry) Zhou, 7/11/15

function [slabs,holes,fixedBounds,decayFunc] = generateSlabHoleNanobeamHalfCrossSubdom_roughBeam(P)

a = P.a;
hh = P.hh;
hw = P.hw;
w = P.w;
mDef = P.mDef;
oblong = P.oblong;
nDef = P.nDef;
nNom = P.nNom;
useSym = P.useSym;
stdDev = P.stdDev;
stdDevPos = P.stdDevPos;
stdDevRoughBeam = P.stdDevRoughBeam;
roughBeamSections = P.roughBeamSections;
aC = P.aC;
hC = P.hC;
wC = P.wC;
nC = P.nC;
wS = P.wS;
xC = P.xC;
yC = P.yC;
if isfield(P,'PMLLen')
    PMLLen = P.PMLLen;
else
    PMLLen = 4e-6;
end
if isfield(P,'PMLStart')
    %PMLStart = max(P.PMLStart+aC*(nC+1),1.5*aC*(nC+2)); % for a circular
    %edge
    PMLStart = P.PMLStart+aC*(nC+1);
else
    %PMLStart = 1.5*aC*(nC+2);
end
if useSym(2) == 1 || useSym(1) == 1
    disp('x, y-symmetry not supported for half crosses! Results may be incorrect!')
end
PatchStart = PMLStart;  % starting position of coarse mesh region

% Set up positions and hole sizes for nanobeam
fs = [1-mDef*(1-((0:nDef-1)/nDef).^2),ones(1,nNom)];
if ~useSym(1)
    fs = [fliplr(fs(2:end)),fs];
end
as = a*(fs(1:end-1)+fs(2:end))/2;
hhs = hh*fs.^(1-oblong);
hws = hw*fs.^(1+oblong);
if (P.useFittedCavity)
    hhs(P.fittedindices)=P.fittedCavityhh;
    hws(P.fittedindices)=P.fittedCavityhw;
end
x = [0 cumsum(as)];
y = zeros(1,length(x));
beamLen = max(x)+2*a;
if ~useSym(1)
    x = x-max(x)/2;
end

% Create the beam, and air slabs
slab.type = 'Rectangle';
slab.wx = beamLen;
if useSym(1)
    slab.x = beamLen/2;
else
    slab.x = 0;
end
slab.wy = w;
slab.y = 0;
slabs = {slab};

slab.type = 'Air';
slab.wy = w+4e-6;
slab.y = 0;
slabs{end+1} = slab;

% Create the external slab on the right
slab.type = 'Rectangle';
slab.wx = wS;
slab.x = beamLen/2+wS/2;
slab.wy = PatchStart;
slab.y = (PatchStart-w)/2;
slabs{end+1} = slab;

% Create PML slab on the right
slab.type = 'PML';
slab.wx = wS;
slab.x = beamLen/2+wS/2;
slab.wy = PMLLen;
slab.y = PatchStart-w/2+PMLLen/2;
slabs{end+1} = slab;

FuncR = ['((x>0).*(y>',num2str(PMLStart),').*((y-',num2str(PMLStart),')./',num2str(P.a),').^2)'];

% Create the external slab on the left, for now for simplicity of boundary
% conditions I am using a rectangular outside region
slab.type = 'Rectangle';
slab.wx = PatchStart;
slab.x = -beamLen/2-slab.wx/2;
slab.wy = PatchStart;
slab.y = (PatchStart-w)/2;
slabs{end+1} = slab;

% Create PML slabs on the left
slab.type = 'PML';
slab.wx = PMLStart+PMLLen-PatchStart;
slab.x = -beamLen/2-PatchStart-slab.wx/2;
slab.wy = PMLStart+PMLLen;
slab.y = (slab.wy-w)/2;
slabs{end+1} = slab;

slab.type = 'PML';
slab.wx = PMLStart+PMLLen;
slab.x = -beamLen/2-slab.wx/2;
slab.wy = PMLStart+PMLLen-PatchStart;
slab.y = PatchStart-w/2+slab.wy/2;
slabs{end+1} = slab;

% Create PML Cut
slab.type = 'PMLcut';
slab.wx = beamLen*5;
slab.x = 0;
slab.wy = (PMLStart+PMLLen-w/2)*2;
slab.y = 0;
slabs{end+1} = slab;


% Create rough Beam
slab.type = 'roughBeam';

slab.wx=[];
slab.wy=[];

slab.x = -beamLen/2;
slab.y = w/2;

for k=-(roughBeamSections/2 - 1):1:0

    xtemp = k/(roughBeamSections/2)*beamLen/2;
    ytemp = w/2;
    [xtemp,ytemp] = randomizePos(xtemp,ytemp,stdDevRoughBeam);
    slab.x(end+1) = xtemp;
    slab.y(end+1) = ytemp;
end

for k=1:(roughBeamSections/2 - 1)

    xtemp = k/(roughBeamSections/2)*beamLen/2;
    ytemp = w/2;
    [xtemp,ytemp] = randomizePos(xtemp,ytemp,stdDevRoughBeam);
    slab.x(end+1) = xtemp;
    slab.y(end+1) = ytemp;
end
slab.x(end+1) = beamLen/2;
slab.y(end+1) = w/2;

slab.x(end+1) = beamLen/2;
slab.y(end+1) = -w/2;

for k=(roughBeamSections/2 - 1):-1:0

    xtemp = k/(roughBeamSections/2)*beamLen/2;
    ytemp = -w/2;
    [xtemp,ytemp] = randomizePos(xtemp,ytemp,stdDevRoughBeam);
    slab.x(end+1) = xtemp;
    slab.y(end+1) = ytemp;
end

for k=-1:-1:-(roughBeamSections/2 - 1)

    xtemp = k/(roughBeamSections/2)*beamLen/2;
    ytemp = -w/2;
    [xtemp,ytemp] = randomizePos(xtemp,ytemp,stdDevRoughBeam);
    slab.x(end+1) = xtemp;
    slab.y(end+1) = ytemp;
end

slab.x(end+1) = -beamLen/2;
slab.y(end+1) = -w/2;


slabs{end+1} = slab;


FuncL = ['((x<-',num2str(beamLen/2),').*((x+',num2str(beamLen),'/2).^2+(y+',num2str(w),'/2).^2>',...
    num2str(PMLStart),'.^2).*((sqrt((x+',num2str(beamLen),'/2).^2+(y+',num2str(w),'/2).^2)-',...
    num2str(PMLStart),')./(',...
    num2str(P.a),')).^2)'];

decayFunc = ['(1./(1+1i*',num2str(P.mechPMLStr),'*(',FuncL,'+',FuncR,'))).^2'];

% Add elliptical holes
holes = {};
hole.type = 'Ellipse';
for k = 1:length(hhs)
    [hole.x,hole.y] = randomizePos(x(k),y(k),stdDevPos);
    hole.rx = randomize(hhs(k)/2,stdDev(1));
    hole.ry = randomize(hws(k)/2,stdDev(2));
    holes{end+1} = hole;
end

% Add cross holes
clear hole;
hole.type = 'Cross';
xH = beamLen/2+xC;
if useSym(1)
    xH = beamLen+xC;
end
yH = yC-w/2;
for xInd = 1:nC+1
    for yInd = 1:nC+1
        if ~(xInd==1 && yInd ==1)   % don't make central hole
            [hole.x,hole.y] = randomizePos(xH,yH,stdDevPos);
            hole.h1 = randomize(hC,stdDev(3));
            hole.h2 = randomize(hC,stdDev(3));
            hole.w1 = randomize(wC,stdDev(4));
            hole.w2 = randomize(wC,stdDev(4));
            holes{end+1} = hole;
            if ~useSym(1)
                [hole.x,hole.y] = randomizePos(-xH,yH,stdDevPos);
                hole.h1 = randomize(hC,stdDev(3));
                hole.h2 = randomize(hC,stdDev(3));
                hole.w1 = randomize(wC,stdDev(4));
                hole.w2 = randomize(wC,stdDev(4));
                holes{end+1} = hole;
            end 
        end
        yH = yH+aC;
    end
    xH = xH+aC;
    yH = yC-w/2;
end

%
fixedBounds = {[beamLen/2+wS/2,PMLStart+PMLLen-w/2,0],[0,1,0],...
    [-beamLen/2-PMLStart/2-PMLLen/2,PMLStart+PMLLen-w/2,0],[0,1,0]};
fixedBounds{end+1} = [-beamLen/2-PMLStart-PMLLen,0,0];
fixedBounds{end+1} = [1,0,0];


%{
if useSym(1)
    startPML = [Inf,aC*(nC+1)];
else
    startPML = [Inf,aC*(nC+1)];
end
%}