% This function runs a simulation for the mechanical modes of a photonic crystal cavity.
% The input structure P is assumed to have the following entries (plus additional
% geometry parameters, see createCavity.m):
%
% P.neigs = # of eigenvalues to find; P.freq = target frequency;
% P.useSym/sym = which planes have symmetry, and what type of symmetry;
% IF ISOTROPIC:
% P.E = Young's modulus of the material (isotropic); P.nu = Poisson's ratio of the material;
% IF ANISOTROPIC:
% P.D = vector containing elements of anisotropic stiffness tensor
% WHEN BOTH ARE PRESENT ANISOTROPY WILL BE PRIORITIZED
% P.rho = density of the material;
%
% PML ARGUMENTS ARE OPTIONAL. IF NOT PRESENT WE DEFAULT TO NO PML
% P.mechPML = whether or not to use a mechanical PML; P.mechPMLStr = strength of PML;
%
% varargin = additional arguments, submitted as 'variableName',value pairs;
%            supported arguments are:
%               maxDoF = maximum # of degrees of freedom for the simulation;
%               lambdaMesh = "wavelength" for determining max. mesh size
%               lambdaDiv = "wavelength" division factor for max. mesh size
%               meshQuality = starting 'hauto' param for mesh quality
%               meshSweep = 1 if using swept mesh
%
% Sean Meenehan, 1/7/15

function ds = runMech(P,varargin)


flclear fem

useSym = P.useSym;
sym = P.sym;
rho = P.rho;
neigs = P.neigs;
freq = P.freq;
thi = P.thi;

% Parse varargin
maxDoF = 4e6;
lambdaMesh = 1550e-9;
lambdaDiv = 15;
meshQuality = 1;
meshSweep = 1;
sweeplayers = 1;
filename = '';
if(mod(length(varargin),2)>0); error('Incorrect parameter specification.'); end;
for k=1:2:length(varargin)
    if (strcmp(varargin{k},'filename'))
        filename = varargin{k+1};
    elseif (exist(varargin{k},'var'))
        eval([varargin{k},' = ',num2str(varargin{k+1}),';']);
    else
        display([varargin{k} ' is not a valid parameter.']);
    end
end

%% Set up geometry
clear draw
draw.s.objs = {};
draw.s.name = {};
draw.s.tags = {};
fem.draw = draw;
[fem,P,fixedBounds,startPML] = createCavity(fem,P,'useAir',0);
% note that in the case of NanobeamCross and NanobeamHalfCross, startPML
% actually returns the decay function
fem.geom = geomcsg(fem);
ds.P = P;

%% Set up application mode
clear appl
appl.mode.class = 'SmeSolid3';
appl.module = 'MEMS';
appl.gporder = 4;
appl.cporder = 2;
appl.assignsuffix = '_smsld';
clear prop
prop.analysis='eigen';
appl.prop = prop;

%% Set up boundary conditions
clear bnd
bnd.constrcond = {'free','fixed','sym','antisym'};

% Initialize all boundaries as free
nfaces = geominfo(fem.geom,'out','no','Od',2); % # of boundaries in the structure   
bnd.ind = ones(1,nfaces);

% Fixed boundary condition
bndFix = [];
for k = 1:2:length(fixedBounds)
    aux = bndindex(fem.geom,fixedBounds{k},fixedBounds{k+1});
    bndFix(end+1:end+length(aux)) = aux; clear aux
end
bnd.ind(bndFix) = 2;

% For any symmetries used, overwrite the free boundary condition as
% needed
bndSym = [];
bndAntiSym = [];
if useSym(1)
    aux = bndindex(fem.geom,[0 0 0],[1 0 0]);
    if sym(1)
        bndSym(end+1:end+length(aux)) = aux; clear aux
    else
        bndAntiSym(end+1:end+length(aux)) = aux; clear aux
    end
end
if useSym(2)
    aux = bndindex(fem.geom,[0 0 0],[0 1 0]);
    if sym(2)
        bndSym(end+1:end+length(aux)) = aux; clear aux
    else
        bndAntiSym(end+1:end+length(aux)) = aux; clear aux
    end
end
if useSym(3)
    aux = bndindex(fem.geom,[0 0 0],[0 0 1]);
    if sym(3)
        bndSym(end+1:end+length(aux)) = aux; clear aux
    else
        bndAntiSym(end+1:end+length(aux)) = aux; clear aux
    end
end
bnd.ind(bndSym) = 3;
bnd.ind(bndAntiSym) = 4;

appl.bnd = bnd;
% IF ISOTROPIC:
% P.E = Young's modulus of the material (isotropic); P.nu = Poisson's ratio of the material;
% IF ANISOTROPIC:
% P.D = vector containing elements of anisotropic stiffness tensor
% WHEN BOTH ARE PRESENT ANISOTROPY WILL BE PRIORITIZED
% P.rho = density of the material;
%
% PML ARGUMENTS ARE OPTIONAL. IF NOT PRESENT WE DEFAULT TO NO PML
% P.mechPML = whether or not to use a mechanical PML; P.mechPMLStr = strength of PML

%% Set up material properties
clear equ
equ.rho = {rho};

% Create PML function, if needed
% Note: In Sean's previous implementation the abs(x) parts were incorrect.
% I have changed this to the same as Amir (sims/ev_coup/201206_backup/solvers/solve_mechanical.m)
decayFunc = '1';
if isfield(P,'mechPML')
    if P.mechPML == 1
        if strcmp(P.type,'NanobeamHalfCross')
            decayFunc = startPML;
        elseif strcmp(P.type,'NanobeamCross')
            decayFunc = startPML;
        else
            decayFunc = ['(1./(1+1i*',num2str(P.mechPMLStr),'*((abs(x)>',num2str(startPML(1)),').*', ...
                     '((abs(x)-',num2str(startPML(1)),')/',num2str(P.a),').^2',...
                     '+(abs(y)>',num2str(startPML(2)),').*'...
                     '((abs(y)-',num2str(startPML(2)),')/',num2str(P.a),').^2))).^2'];
%             decayFunc = ['(1./(1+1i*',num2str(P.mechPMLStr),'*((abs(y)>',num2str(startPML(2)),').*'...
%                      '((abs(y)-',num2str(startPML(2)),')/',num2str(P.a),').^2))).^2'];
        end
    end
end

% Choose anistotropic or isotropic material, as needed
if isfield(P,'D')
    if length(P.D) < 21
        error('Insufficient number of stiffness constants specified');
    end
    equ.materialmodel = 'aniso';
    equ.D = {{[num2str(P.D(1)),'*',decayFunc],[num2str(P.D(2)),'*',decayFunc], ...
              [num2str(P.D(3)),'*',decayFunc],[num2str(P.D(4)),'*',decayFunc], ...
              [num2str(P.D(5)),'*',decayFunc],[num2str(P.D(6)),'*',decayFunc]; ...
              [num2str(P.D(2)),'*',decayFunc],[num2str(P.D(7)),'*',decayFunc], ...
              [num2str(P.D(8)),'*',decayFunc],[num2str(P.D(9)),'*',decayFunc], ...
              [num2str(P.D(10)),'*',decayFunc],[num2str(P.D(11)),'*',decayFunc]; ...
              [num2str(P.D(3)),'*',decayFunc],[num2str(P.D(8)),'*',decayFunc], ...
              [num2str(P.D(12)),'*',decayFunc],[num2str(P.D(13)),'*',decayFunc], ...
              [num2str(P.D(14)),'*',decayFunc],[num2str(P.D(15)),'*',decayFunc]; ...
              [num2str(P.D(4)),'*',decayFunc],[num2str(P.D(9)),'*',decayFunc], ...
              [num2str(P.D(13)),'*',decayFunc],[num2str(P.D(16)),'*',decayFunc], ...
              [num2str(P.D(17)),'*',decayFunc],[num2str(P.D(18)),'*',decayFunc]; ...
              [num2str(P.D(5)),'*',decayFunc],[num2str(P.D(10)),'*',decayFunc], ...
              [num2str(P.D(14)),'*',decayFunc],[num2str(P.D(17)),'*',decayFunc], ...
              [num2str(P.D(19)),'*',decayFunc],[num2str(P.D(20)),'*',decayFunc]; ...
              [num2str(P.D(6)),'*',decayFunc],[num2str(P.D(11)),'*',decayFunc], ...
              [num2str(P.D(15)),'*',decayFunc],[num2str(P.D(18)),'*',decayFunc], ...
              [num2str(P.D(20)),'*',decayFunc],[num2str(P.D(21)),'*',decayFunc]}};
elseif (isfield(P,'E') && isfield(P,'nu')) 
    equ.nu = {P.nu};
    equ.E = {[num2str(P.E),'*',decayFunc]};
else
    error('No stiffness type specified');
end
equ.ind = 1;
appl.equ = equ;

fem.appl{1} = appl;
fem.border = 1;
fem=multiphysics(fem);

%
%% Solve problem with best mesh possible
ok=0;
if useSym(3)
    botBnd = bndindex(fem.geom,[0,0,0],[0,0,1]);
else
    botBnd = bndindex(fem.geom,[0,0,-thi/2],[0,0,1]);
end
topBnd = bndindex(fem.geom,[0,0,thi/2],[0,0,1]);
display(['Meshing with quality: ',num2str(meshQuality)]);
if meshSweep
    fem.mesh=meshsweep(fem,'hauto',meshQuality,'hmax',lambdaMesh/lambdaDiv, ...
                       'sourceface',{1,botBnd},'targetface',{1,topBnd},'Elsweeplayers',{1,sweeplayers});            
else
    fem.mesh=meshinit(fem,'hauto',meshQuality,'hmax',lambdaMesh/lambdaDiv);
end
fem.xmesh=meshextend(fem);
dofs = xmesh_dofs(fem);
display(['Estimated # of DoFs: ',num2str(dofs)]);
%
while ok==0 && meshQuality<10
    while dofs>=maxDoF && meshQuality<9
        meshQuality = meshQuality+1;
        display(['Meshing with quality: ',num2str(meshQuality)]);
        if meshSweep
            fem.mesh=meshsweep(fem,'hauto',meshQuality,'hmax',lambdaMesh/lambdaDiv, ...
                               'sourceface',{1,botBnd},'targetface',{1,topBnd},'Elsweeplayers',{1,sweeplayers});

        else
            fem.mesh=meshinit(fem,'hauto',meshQuality,'hmax',lambdaMesh/lambdaDiv);
        end
        fem.xmesh=meshextend(fem);
        dofs = xmesh_dofs(fem);
        display(['Estimated # of DoFs: ',num2str(dofs)]);
    end
    pause(2);
    try
        % Solve problem
        display('Solving');
        fem.sol=femeig(fem,'solcomp',{'w','v','u'},'outcomp',{'w','v','u'}, ...
                       'shift',0-1i*2*pi*freq,'neigs',neigs);
        display('Done');
        ok = 1; 
    catch        
        display(lasterr)
        meshQuality = meshQuality + 1;
        display(['Meshing with quality: ',num2str(meshQuality)]);
        if meshSweep
            fem.mesh=meshsweep(fem,'hauto',meshQuality,'hmax',lambdaMesh/lambdaDiv, ...
                               'sourceface',{1,botBnd},'targetface',{1,topBnd},'Elsweeplayers',{1,sweeplayers});
        else
            fem.mesh=meshinit(fem,'hauto',meshQuality,'hmax',lambdaMesh/lambdaDiv);
        end
        fem.xmesh=meshextend(fem);
        dofs = xmesh_dofs(fem);
        display(['Estimated # of DoFs: ',num2str(dofs)]);
    end
end

%% Post-processing
ds.freqs = abs(imag(fem.sol.lambda)/(2*pi));
if isfield(P,'mechPML')
    if P.mechPML == 1
        ds.decay = 2*abs(real(fem.sol.lambda/(2*pi)));
        [~,minsol] = min(ds.decay); % localized mode has lowest decay
        ds.Q = ds.freqs(minsol)/ds.decay(minsol);
        ds.minsol = minsol;
        disp('Q value is:')
        disp(ds.Q)
        disp('Frequency is:')
        disp(ds.freqs(minsol))
    end
end
%}
ds.fem = fem;
%



%cd 'MechSim/PML_HalfCross'
%{
filenum = 1;
savefilename = strcat(date,'_',num2str(filenum),'.mat');
while exist(savefilename,'file')
    filenum = filenum + 1;
    savefilename = strcat(date,'_',num2str(filenum),'.mat');
end
if ~strcmp(filename,'')
    savefilename = strcat(filename,'.mat');
    save(savefilename,'ds')
else
    save(savefilename,'ds')
end
%}

%{
% Plot solution
figure;
h1 = postplot(fem, ...
         'tetdata',{'(disp_smsld)','cont','internal'}, ...
         'tetmap','Rainbow', ...
         'tetkeep',1, ...
         'tetkeeptype','random', ...
         'deformbnd',{'u','v','w'}, ...
         'deformscale',1e-8, ...
         'deformsub',{'u','v','w'}, ...
         'deformscale',1e-8, ...
         'solnum',minsol, ...
         'title',strcat(filename,' Q=',num2str(ds.Q),' phase=0 (log(q))'),...
         'geom','off', ...
         'grid','on', ...
         'phase',0,...
         'campos',[0,0,3E-5], ...
         'camtarget',[0,0,0], ...
         'camup',[0,1,0], ...
         'camva',11);
% if ~strcmp(filename,'')
%     fig1name = strcat(filename,'_ph0');
%     saveas(gcf,fig1name,'fig')
% end
% figure;
% h2 = postplot(fem, ...
%          'tetdata',{'(disp_smsld)','cont','internal'}, ...
%          'tetmap','Rainbow', ...
%          'tetkeep',1, ...
%          'tetkeeptype','random', ...
%          'solnum', minsol, ...
%          'phase',(90)*pi/180, ...
%          'title',strcat(filename,' Q=',num2str(ds.Q),' phase=90 (log(q))'),...
%          'grid','on', ...
%          'campos',[0,0,3E-5], ...
%          'camtarget',[0,0,0], ...
%          'camup',[0,1,0], ...
%          'camva',11);
% if ~strcmp(filename,'')
%     fig2name = strcat(filename,'_ph90');
%     saveas(gcf,fig2name,'fig')
% end
%}
%cd ..
%cd ..
%

