% This function computes the mechanical band structure for a given unit
% cell geometry. Only the bands along the edge of the first IBZ are
% computed. THe input structure P is assumed to have the following fields,
% plus additional fields defined in createUnitCell.m:
%
% P.neigs = # of bands to find; P.freq = frequency at which to start looking for bands;
% P.kPts = # of k-points interpolated between each high-symmetry point;
% P.useSym/sym = which planes have symmetry, and what type of symmetry;
% P.maxDoF = maximum # of degrees of freedom for the simulation;
% P.thi = thickness of slab; P.rho = density of the material;
% IF ISOTROPIC:
% P.E = Young's modulus of the material (isotropic); P.nu = Poisson's ratio of the material;
% IF ANISOTROPIC:
% P.D = vector containing elements of anisotropic stiffness tensor
% WHEN BOTH ARE PRESENT ANISOTROPY WILL BE PRIORITIZED
%
% Sean Meenehan, 9/11/13

function ds = runMechBands(P)

flclear fem

neigs = P.neigs;
freq = P.freq;
kPts = P.kPts;
useSym = P.useSym;
sym = P.sym;
maxDoF = P.maxDoF;
rho = P.rho;
thi = P.thi;
meshDiv = 40;

% Set up the geometry
clear draw
draw.s.objs = {};
draw.s.name = {};
draw.s.tags = {};
fem.draw = draw;
[fem,P,latticeVecs,fixedBounds] = createUnitCell(fem,P);
fem.geom = geomcsg(fem);
nFaces = geominfo(fem.geom,'out','no','Od',2);
dim = length(latticeVecs);

clear appl
appl.mode.class = 'SmeSolid3';
appl.module = 'SME';
appl.gporder = 4;
appl.cporder = 2;
appl.sshape = 2;
appl.assignsuffix = '_smsld';
clear prop
prop.analysis='eigen';
appl.prop = prop;
fem.frame = {'ref'};
fem.border = 1;
clear units;
units.basesystem = 'SI';
fem.units = units;

%% Periodic boundary conditions
srcB = {};
dstB = {};
if dim == 1
    srcB{1} = bndindex(fem.geom,[0,0,0],[1,0,0]);
    dstB{1} = bndindex(fem.geom,latticeVecs{1},[1,0,0]);
elseif dim == 2
    a1 = latticeVecs{1};
    a2 = latticeVecs{2};
    n1 = [1,1,0]-a1*dot(a1,[1,1,0])/norm(a1)^2;
    n2 = [1,1,0]-a2*dot(a2,[1,1,0])/norm(a2)^2;
    srcB{1} = bndindex(fem.geom,[0,0,0],n1);
    srcB{2} = bndindex(fem.geom,[0,0,0],n2);
    dstB{1} = bndindex(fem.geom,a2,n1);
    dstB{2} = bndindex(fem.geom,a1+a2,n2);
else
    error('Crystals with dimension greater than 2 not supported');
end

% Vertices adjacent to each boundary
srcV = {};
dstV = {};
adj = geominfo(fem.geom,'out','adj','Odp',[2;0]);
totBnds = 0;
nBnds = zeros(1,dim);
for k = 1:dim
    srcTmp = srcB{k};
    dstTmp = dstB{k};
    totBnds = totBnds+length(srcTmp);
    nBnds(k) = totBnds;
    if length(dstTmp) ~= length(srcTmp)
        error(['Src/dst boundary mismatch in dimension ',num2str(k)]);
    end
    for bndInd = 1:length(srcTmp)
        srcV{end+1} = find(adj{1}(srcTmp(bndInd),:)==1);
        dstV{end+1} = find(adj{1}(dstTmp(bndInd),:)==1);
    end
end

varStrs = ['u','v','w'];

% Extrusion coupling variables
clear elemcpl
clear elem
elem.elem = 'elcplextr';
elem.g = {'1'};

% Set source domains
src = cell(1,1);
clear bnd
bnd.expr = {};
bnd.map = {};
for bndInd = 1:totBnds
    for k = 1:3
        var = varStrs(k);
        cellTmp = {};
        mapTmp = {};
        for m = 1:totBnds+1
            cellTmp{end+1} = {};
            mapTmp{end+1} = '1';
        end
        cellTmp{bndInd} = var;
        bnd.expr{end+1} = cellTmp;
        bnd.map{end+1} = mapTmp;
    end
end

bnd.ind = {};
bloch = [];
for k = 1:dim
    srcTmp = srcB{k};
    bloch(end+1:end+length(srcTmp)) = srcTmp;
    for bndInd = 1:length(srcTmp)
        bnd.ind{end+1} = {num2str(srcTmp(bndInd))};
    end
end

cellTmp={};
nonBloch = 1:nFaces;
nonBloch(bloch) = [];
for k = 1:length(nonBloch)
    cellTmp{end+1} = num2str(nonBloch(k));
end
bnd.ind{end+1} = cellTmp;
src{1} = {{},{},bnd,{}};
elem.src = src;

% Set destination domains
geomdim = cell(1,1);
clear bnd

bnd.map = {};
for bndInd = 1:totBnds
    tmpMap = {{}};
    for k = 1:totBnds
        tmpMap{end+1} = {};
    end
    tmpMap{bndInd+1} = num2str(bndInd+1);
    for k = 1:3
        bnd.map{end+1} = tmpMap;
    end
end

bloch = [];
for k = 1:dim
    dstTmp = dstB{k};
    bloch(end+1:end+length(dstTmp)) = dstTmp;
end

cellTmp={};
nonBloch = 1:nFaces;
nonBloch(bloch) = [];
for k = 1:length(nonBloch)
    cellTmp{end+1} = num2str(nonBloch(k));
end
bnd.ind = {cellTmp};

for k = 1:dim
    dstTmp = dstB{k};
    for bndInd = 1:length(dstTmp)
        bnd.ind{end+1} = {num2str(dstTmp(bndInd))};
    end
end
geomdim{1} = {{},{},bnd,{}};
elem.geomdim = geomdim;

% Coupling variables
elem.var = {};
for bndInd = 1:totBnds
    elem.var{end+1} = ['u',num2str(bndInd)];
    elem.var{end+1} = ['v',num2str(bndInd)];
    elem.var{end+1} = ['w',num2str(bndInd)];
end

% Set source/destination points for the linear transformation between
% domains
map = cell(1,totBnds+1);
clear submap
submap.type = 'unit';
map{1} = submap;

for bndInd = 1:totBnds
    clear submap
    submap.type = 'linear';
    submap.sg = '1';
    % The source points for the transformation are the points in the
    % destination domain, and vice versa (see the elcplextr documentation 
    % in the COMSOL ref. manual)
    cellTmp={};
    srcTmp = srcV{bndInd};
    dstTmp = dstV{bndInd};
    for k=1:length(dstTmp)
        cellTmp{end+1} = num2str(dstTmp(k));
    end
    submap.sv = cellTmp;

    submap.dg = '1';
    cellTmp={};
    for k=1:length(srcTmp)
        cellTmp{end+1} = num2str(srcTmp(k));
    end
    submap.dv = cellTmp;

    map{bndInd+1} = submap;
end
elem.map = map;
elemcpl{1} = elem;


% Point constraint variables (used for periodic conditions)
clear elem
elem.elem = 'elpconstr';
elem.g = {'1'};
clear bnd
bnd.constr = {};
bnd.cpoints = {};
bnd.ind = {};
cPoint = {};
for bndInd = 1:3*totBnds
    cPoint{end+1} = '2';
end
for bndInd = 1:totBnds
    dimNum = 1;
    bndNum = bndInd;
    for kk = 1:dim
        if bndInd <= nBnds(kk)
            dimNum = kk;
            if kk > 1
                bndNum = bndInd-sum(nBnds(kk-1));
            end
            break;
        end
    end
    tmpConstr = {};
    for k = 1:3*totBnds
        tmpConstr{end+1} = '0';
    end
    tmpConstr{(bndInd-1)*3+1} = ['u',num2str(bndInd),'-(u*exp(i*(kx*a',num2str(dimNum),'x+ky*a',num2str(dimNum),'y+kz*a',num2str(dimNum),'z)))'];
    tmpConstr{(bndInd-1)*3+2} = ['v',num2str(bndInd),'-(v*exp(i*(kx*a',num2str(dimNum),'x+ky*a',num2str(dimNum),'y+kz*a',num2str(dimNum),'z)))'];
    tmpConstr{(bndInd-1)*3+3} = ['w',num2str(bndInd),'-(w*exp(i*(kx*a',num2str(dimNum),'x+ky*a',num2str(dimNum),'y+kz*a',num2str(dimNum),'z)))'];
    bnd.constr{end+1} = tmpConstr;
    bnd.cpoints{end+1} = cPoint;
    bnd.ind{end+1} = {num2str(dstB{dimNum}(bndNum))};
end
elem.geomdim = {{{},{},bnd,{}}};
elemcpl{2} = elem;


% Vector constraint variables (used for periodic conditions)
clear elem
elem.elem = 'elsconstr';
elem.g = {'1'};
elem.geomdim = {{{},{},{},{}}};
elemcpl{3} = elem;
fem.elemcpl = elemcpl;

%% Symmetry and fixed boundary conditions
clear bnd
bnd.constrcond = {'free','fixed','sym','antisym'};

% Initialize all boundaries as free
bnd.ind = ones(1,nFaces);

% Fixed boundary condition
bndFix = [];
for k = 1:2:length(fixedBounds)
    aux = bndindex(fem.geom,fixedBounds{k},fixedBounds{k+1});
    bndFix(end+1:end+length(aux)) = aux; clear aux
end
bnd.ind(bndFix) = 2;

% For any symmetries used, overwrite the scattering boundary condition as
% needed
bndSym = [];
bndAntiSym = [];
if useSym(1)
    aux = bndindex(fem.geom,[0 0 0],[1 0 0]);
    if sym(1)
        bndSym(end+1:end+length(aux)) = aux; clear aux
    else
        bndAntiSym(end+1:end+length(aux)) = aux; clear aux
    end
end
if useSym(2)
    aux = bndindex(fem.geom,[0 0 0],[0 1 0]);
    if sym(2)
        bndSym(end+1:end+length(aux)) = aux; clear aux
    else
        bndAntiSym(end+1:end+length(aux)) = aux; clear aux
    end
end
if useSym(3)
    aux = bndindex(fem.geom,[0 0 0],[0 0 1]);
    if sym(3)
        bndSym(end+1:end+length(aux)) = aux; clear aux
    else
        bndAntiSym(end+1:end+length(aux)) = aux; clear aux
    end
end
bnd.ind(bndSym) = 3;
bnd.ind(bndAntiSym) = 4;

appl.bnd = bnd;

fem.const = {'kx','0','ky','0','kz','0'};
for k = 1:dim
    aVec = latticeVecs{k};
    fem.const{end+1} = ['a',num2str(k),'x'];
    fem.const{end+1} = num2str(aVec(1));
    fem.const{end+1} = ['a',num2str(k),'y'];
    fem.const{end+1} = num2str(aVec(2));
    fem.const{end+1} = ['a',num2str(k),'z'];
    fem.const{end+1} = num2str(aVec(3));
end

%% Material properties
clear equ
equ.rho = {rho};
if isfield(P,'D')
    if length(P.D) < 21
        error('Insufficient number of stiffness constants specified');
    end
    equ.materialmodel = 'aniso';
    equ.D = {{P.D(1),P.D(2),P.D(3),P.D(4),P.D(5),P.D(6); ...
              P.D(2),P.D(7),P.D(8),P.D(9),P.D(10),P.D(11); ...
              P.D(3),P.D(8),P.D(12),P.D(13),P.D(14),P.D(15); ...
              P.D(4),P.D(9),P.D(13),P.D(16),P.D(17),P.D(18); ...
              P.D(5),P.D(10),P.D(14),P.D(17),P.D(19),P.D(20); ...
              P.D(6),P.D(11),P.D(15),P.D(18),P.D(20),P.D(21)}};
elseif (isfield(P,'E') && isfield(P,'nu')) 
    equ.nu = {P.nu};
    equ.E = {P.E};
else
    error('No stiffness type specified');
end
equ.ind = 1;
appl.equ = equ;

fem.appl{1} = appl;
fem.border = 1;
fem=multiphysics(fem);

% ODE Settings
clear ode
clear units;
units.basesystem = 'SI';
ode.units = units;
fem.ode = ode;

fem = multiphysics(fem);

%% Meshing
meshQuality = 5;
if useSym(3)
    botBnd = bndindex(fem.geom,[0,0,0],[0,0,1]);
else
    botBnd = bndindex(fem.geom,[0,0,-thi/2],[0,0,1]);
end
topBnd = bndindex(fem.geom,[0,0,thi/2],[0,0,1]);
display(['Meshing with quality: ',num2str(meshQuality)]);
% fem.mesh=meshsweep(fem,'hauto',meshQuality,'sourceface',{1,botBnd},'targetface',{1,topBnd});
fem.mesh=meshinit(fem,'hauto',meshQuality,'hmax',1500e-9/meshDiv);
fem.xmesh=meshextend(fem);
dofs = xmesh_dofs(fem);
display(['Estimated # of DoFs: ',num2str(dofs)]);

while dofs >= maxDoF && meshQuality < 10
    % Initialize mesh
    meshQuality = meshQuality +1;
    display(['Meshing with quality: ',num2str(meshQuality)]);
%     fem.mesh=meshsweep(fem,'hauto',meshQuality,'sourceface',{1,botBnd},'targetface',{1,topBnd});
    fem.mesh=meshinit(fem,'hauto',meshQuality,'hmax',1500e-9/meshDiv);
    fem.xmesh=meshextend(fem);
    dofs = xmesh_dofs(fem);
    display(['Estimated # of DoFs: ',num2str(dofs)]);
end


%% Sweep k-points and solve
if dim == 1
    kPtsX = [0 pi/latticeVecs{1}(1)];
    kPtsY = [0 0];
    kPtsZ = [0 0];
elseif dim == 2
    a1 = latticeVecs{1};
    a2 = latticeVecs{2};
    a3 = [0,0,1];
    b1 = 2*pi*cross(a2,a3)/dot(a1,cross(a2,a3));
    b2 = 2*pi*cross(a3,a1)/dot(a1,cross(a2,a3));
    kPtsX = [0 b1(1)/2];
    kPtsY = [0 b1(2)/2];
    n1 = [1,1,0]-b1*dot(b1,[1,1,0])/dot(b1,b1);
    n2 = [1,1,0]-b2*dot(b2,[1,1,0])/dot(b2,b2);
    X = linsolve([n1',n2'],0.5*(b2'-b1'));
    kPtsX(end+1:end+2) = [b1(1)/2+X(1)*n1(1) 0];
    kPtsY(end+1:end+2) = [b1(2)/2+X(1)*n1(2) 0];
    kPtsZ = [0 0 0 0];
else
    % Shouldn't reach
    error('Dim > 2');
end

nK = 0;
for k = 2:length(kPtsX)
    dx = kPtsX(k)-kPtsX(k-1);
    dy = kPtsY(k)-kPtsY(k-1);
    dz = kPtsZ(k)-kPtsZ(k-1);
	
    nStart = 1;
    if k > 1
        nStart = 0;
    end

    for n = nStart:kPts
        kx = kPtsX(k-1)+dx*n/kPts;
        ky = kPtsY(k-1)+dy*n/kPts;
        kz = kPtsZ(k-1)+dz*n/kPts;
		nK = nK+1;
		ds.kx(nK) = kx;
		ds.ky(nK) = ky;
        ds.kz(nK) = kz;
    end
end
ds.nK = nK;
ds.P = P;

for n = 1:nK
    display(['Solving k-point ' num2str(n)]);
    fem.const{2*strmatch('kx',fem.const(1:2:end),'exact')} = num2str(ds.kx(n));
    if dim > 1
        fem.const{2*strmatch('ky',fem.const(1:2:end),'exact')} = num2str(ds.ky(n));
    end
    if dim > 2
        fem.const{2*strmatch('kz',fem.const(1:2:end),'exact')} = num2str(ds.kz(n));
    end
    fem=multiphysics(fem);
    fem.xmesh=meshextend(fem);

    % For some bizzare reason occasionally the eigensolver will return a
    % number of eigenvalues different from the value passed to neigs. Not
    % sure why, but in order to prevent this from crashing the script when
    % assigning to ds.F, we repeat solving until we have the appropriate
    % number of solutions.
    nsols = 0;
    while nsols ~= neigs
        fem.sol=femeig(fem, ...
                   'conjugate','on', ...
                   'solcomp',{'w','v','u'}, ...
                   'outcomp',{'w','v','u'}, ...
                   'neigs',neigs, ...
                   'linsolver','spooles', ...
                   'shift',-1i*2*pi*freq);
        nsols = length(fem.sol.lambda);
    end

    ds.F(n,:) = abs(fem.sol.lambda)/2/pi;
end
ds.fem = fem;
