% This function computes the mechanical modes at a specified k-point for a given
% unit cell geometry. The input structure P is assumed to have the following 
% fields, plus additional fields defined in createUnitCell.m:
%
% P.neigs = # of bands to find; P.freq = frequency at which to start looking for bands;
% P.kPt = 3 element vector containing desired k-point;
% P.useSym/sym = which planes have symmetry, and what type of symmetry;
% P.thi = thickness of slab; P.rho = density of the material;
% IF ISOTROPIC:
% P.E = Young's modulus of the material (isotropic); P.nu = Poisson's ratio of the material;
% IF ANISOTROPIC:
% P.D = vector containing elements of anisotropic stiffness tensor
% WHEN BOTH ARE PRESENT ANISOTROPY WILL BE PRIORITIZED
%
% varargin = additional arguments, submitted as 'variableName',value pairs;
%            supported arguments are:
%               maxDoF = maximum # of degrees of freedom for the simulation;
%               lambdaMesh = "wavelength" for determining max. mesh size
%               lambdaDiv = "wavelength" division factor for max. mesh size
%               meshQuality = starting 'hauto' param for mesh quality
%               meshSweep = 1 if using swept mesh
%
% Sean Meenehan, 1/7/14

function ds = runMechKPt(P,varargin)

flclear fem

neigs = P.neigs;
freq = P.freq;
kPt = P.kPt;
useSym = P.useSym;
sym = P.sym;
rho = P.rho;
thi = P.thi;

% Parse varargin
maxDoF = 2e6;
lambdaMesh = 1550e-9;
lambdaDiv = 40;
meshQuality = 3;
meshSweep = 0;
if(mod(length(varargin),2)>0); error('Incorrect parameter specification.'); end;
for k=1:2:length(varargin)
    if (exist(varargin{k},'var'))
        eval([varargin{k},' = ',num2str(varargin{k+1}),';']);
    else 
        display([varargin{k} ' is not a valid parameter.']);
    end
end

%% Set up geometry
clear draw
draw.s.objs = {};
draw.s.name = {};
draw.s.tags = {};
fem.draw = draw;
[fem,P,latticeVecs,fixedBounds] = createUnitCell(fem,P);
fem.geom = geomcsg(fem);
nFaces = geominfo(fem.geom,'out','no','Od',2);
dim = length(latticeVecs);

%% Set up application mode
clear appl
appl.mode.class = 'SmeSolid3';
appl.module = 'SME';
appl.gporder = 4;
appl.cporder = 2;
appl.sshape = 2;
appl.assignsuffix = '_smsld';
clear prop
prop.analysis='eigen';
appl.prop = prop;
fem.frame = {'ref'};
fem.border = 1;
clear units;
units.basesystem = 'SI';
fem.units = units;

%% Set up periodic boundary conditions
srcB = {};
dstB = {};
if dim == 1
    srcB{1} = bndindex(fem.geom,[0,0,0],[1,0,0]);
    dstB{1} = bndindex(fem.geom,latticeVecs{1},[1,0,0]);
elseif dim == 2
    a1 = latticeVecs{1};
    a2 = latticeVecs{2};
    n1 = [1,1,0]-a1*dot(a1,[1,1,0])/norm(a1)^2;
    n2 = [1,1,0]-a2*dot(a2,[1,1,0])/norm(a2)^2;
    srcB{1} = bndindex(fem.geom,[0,0,0],n1);
    srcB{2} = bndindex(fem.geom,[0,0,0],n2);
    dstB{1} = bndindex(fem.geom,a2,n1);
    dstB{2} = bndindex(fem.geom,a1+a2,n2);
else
    error('Crystals with dimension greater than 2 not supported');
end

% Vertices adjacent to each boundary
srcV = {};
dstV = {};
adj = geominfo(fem.geom,'out','adj','Odp',[2;0]);
totBnds = 0;
nBnds = zeros(1,dim);
for k = 1:dim
    srcTmp = srcB{k};
    dstTmp = dstB{k};
    totBnds = totBnds+length(srcTmp);
    nBnds(k) = totBnds;
    if length(dstTmp) ~= length(srcTmp)
        error(['Src/dst boundary mismatch in dimension ',num2str(k)]);
    end
    for bndInd = 1:length(srcTmp)
        srcV{end+1} = find(adj{1}(srcTmp(bndInd),:)==1);
        dstV{end+1} = find(adj{1}(dstTmp(bndInd),:)==1);
    end
end

varStrs = ['u','v','w'];

% Extrusion coupling variables
clear elemcpl
clear elem
elem.elem = 'elcplextr';
elem.g = {'1'};

% Set source domains
src = cell(1,1);
clear bnd
bnd.expr = {};
bnd.map = {};
for bndInd = 1:totBnds
    for k = 1:3
        var = varStrs(k);
        cellTmp = {};
        mapTmp = {};
        for m = 1:totBnds+1
            cellTmp{end+1} = {};
            mapTmp{end+1} = '1';
        end
        cellTmp{bndInd} = var;
        bnd.expr{end+1} = cellTmp;
        bnd.map{end+1} = mapTmp;
    end
end

bnd.ind = {};
bloch = [];
for k = 1:dim
    srcTmp = srcB{k};
    bloch(end+1:end+length(srcTmp)) = srcTmp;
    for bndInd = 1:length(srcTmp)
        bnd.ind{end+1} = {num2str(srcTmp(bndInd))};
    end
end

cellTmp={};
nonBloch = 1:nFaces;
nonBloch(bloch) = [];
for k = 1:length(nonBloch)
    cellTmp{end+1} = num2str(nonBloch(k));
end
bnd.ind{end+1} = cellTmp;
src{1} = {{},{},bnd,{}};
elem.src = src;

% Set destination domains
geomdim = cell(1,1);
clear bnd

bnd.map = {};
for bndInd = 1:totBnds
    tmpMap = {{}};
    for k = 1:totBnds
        tmpMap{end+1} = {};
    end
    tmpMap{bndInd+1} = num2str(bndInd+1);
    for k = 1:3
        bnd.map{end+1} = tmpMap;
    end
end

bloch = [];
for k = 1:dim
    dstTmp = dstB{k};
    bloch(end+1:end+length(dstTmp)) = dstTmp;
end

cellTmp={};
nonBloch = 1:nFaces;
nonBloch(bloch) = [];
for k = 1:length(nonBloch)
    cellTmp{end+1} = num2str(nonBloch(k));
end
bnd.ind = {cellTmp};

for k = 1:dim
    dstTmp = dstB{k};
    for bndInd = 1:length(dstTmp)
        bnd.ind{end+1} = {num2str(dstTmp(bndInd))};
    end
end
geomdim{1} = {{},{},bnd,{}};
elem.geomdim = geomdim;

% Coupling variables
elem.var = {};
for bndInd = 1:totBnds
    elem.var{end+1} = ['u',num2str(bndInd)];
    elem.var{end+1} = ['v',num2str(bndInd)];
    elem.var{end+1} = ['w',num2str(bndInd)];
end

% Set source/destination points for the linear transformation between
% domains
map = cell(1,totBnds+1);
clear submap
submap.type = 'unit';
map{1} = submap;

for bndInd = 1:totBnds
    clear submap
    submap.type = 'linear';
    submap.sg = '1';
    % The source points for the transformation are the points in the
    % destination domain, and vice versa (see the elcplextr documentation 
    % in the COMSOL ref. manual)
    cellTmp={};
    srcTmp = srcV{bndInd};
    dstTmp = dstV{bndInd};
    for k=1:length(dstTmp)
        cellTmp{end+1} = num2str(dstTmp(k));
    end
    submap.sv = cellTmp;

    submap.dg = '1';
    cellTmp={};
    for k=1:length(srcTmp)
        cellTmp{end+1} = num2str(srcTmp(k));
    end
    submap.dv = cellTmp;

    map{bndInd+1} = submap;
end
elem.map = map;
elemcpl{1} = elem;


% Point constraint variables (used for periodic conditions)
clear elem
elem.elem = 'elpconstr';
elem.g = {'1'};
clear bnd
bnd.constr = {};
bnd.cpoints = {};
bnd.ind = {};
cPoint = {};
for bndInd = 1:3*totBnds
    cPoint{end+1} = '2';
end
for bndInd = 1:totBnds
    dimNum = 1;
    bndNum = bndInd;
    for kk = 1:dim
        if bndInd <= nBnds(kk)
            dimNum = kk;
            if kk > 1
                bndNum = bndInd-sum(nBnds(kk-1));
            end
            break;
        end
    end
    tmpConstr = {};
    for k = 1:3*totBnds
        tmpConstr{end+1} = '0';
    end
    tmpConstr{(bndInd-1)*3+1} = ['u',num2str(bndInd),'-(u*exp(i*(kx*a',num2str(dimNum),'x+ky*a',num2str(dimNum),'y+kz*a',num2str(dimNum),'z)))'];
    tmpConstr{(bndInd-1)*3+2} = ['v',num2str(bndInd),'-(v*exp(i*(kx*a',num2str(dimNum),'x+ky*a',num2str(dimNum),'y+kz*a',num2str(dimNum),'z)))'];
    tmpConstr{(bndInd-1)*3+3} = ['w',num2str(bndInd),'-(w*exp(i*(kx*a',num2str(dimNum),'x+ky*a',num2str(dimNum),'y+kz*a',num2str(dimNum),'z)))'];
    bnd.constr{end+1} = tmpConstr;
    bnd.cpoints{end+1} = cPoint;
    bnd.ind{end+1} = {num2str(dstB{dimNum}(bndNum))};
end
elem.geomdim = {{{},{},bnd,{}}};
elemcpl{2} = elem;


% Vector constraint variables (used for periodic conditions)
clear elem
elem.elem = 'elsconstr';
elem.g = {'1'};
elem.geomdim = {{{},{},{},{}}};
elemcpl{3} = elem;
fem.elemcpl = elemcpl;

%% Set up symmetry and fixed boundary conditions
clear bnd
bnd.constrcond = {'free','fixed','sym','antisym'};

% Initialize all boundaries as free
bnd.ind = ones(1,nFaces);

% Fixed boundary condition
bndFix = [];
for k = 1:2:length(fixedBounds)
    aux = bndindex(fem.geom,fixedBounds{k},fixedBounds{k+1});
    bndFix(end+1:end+length(aux)) = aux; clear aux
end
bnd.ind(bndFix) = 2;

% For any symmetries used, overwrite the scattering boundary condition as
% needed
bndSym = [];
bndAntiSym = [];
if useSym(1)
    aux = bndindex(fem.geom,[0 0 0],[1 0 0]);
    if sym(1)
        bndSym(end+1:end+length(aux)) = aux; clear aux
    else
        bndAntiSym(end+1:end+length(aux)) = aux; clear aux
    end
end
if useSym(2)
    aux = bndindex(fem.geom,[0 0 0],[0 1 0]);
    if sym(2)
        bndSym(end+1:end+length(aux)) = aux; clear aux
    else
        bndAntiSym(end+1:end+length(aux)) = aux; clear aux
    end
end
if useSym(3)
    aux = bndindex(fem.geom,[0 0 0],[0 0 1]);
    if sym(3)
        bndSym(end+1:end+length(aux)) = aux; clear aux
    else
        bndAntiSym(end+1:end+length(aux)) = aux; clear aux
    end
end
bnd.ind(bndSym) = 3;
bnd.ind(bndAntiSym) = 4;

appl.bnd = bnd;

fem.const = {'kx',num2str(kPt(1))};
if dim > 1
    fem.const{end+1} = 'ky';
    fem.const{end+1} = num2str(kPt(2));
else
    fem.const{end+1} = 'ky';
    fem.const{end+1} = '0';
end
if dim > 2
    fem.const{end+1} = 'kz';
    fem.const{end+1} = num2str(kPt(3));
else
    fem.const{end+1} = 'kz';
    fem.const{end+1} = '0';
end
for k = 1:dim
    aVec = latticeVecs{k};
    fem.const{end+1} = ['a',num2str(k),'x'];
    fem.const{end+1} = num2str(aVec(1));
    fem.const{end+1} = ['a',num2str(k),'y'];
    fem.const{end+1} = num2str(aVec(2));
    fem.const{end+1} = ['a',num2str(k),'z'];
    fem.const{end+1} = num2str(aVec(3));
end

%% Set up material properties
clear equ
equ.rho = {rho};
if isfield(P,'D')
    if length(P.D) < 21
        error('Insufficient number of stiffness constants specified');
    end
    equ.materialmodel = 'aniso';
    equ.D = {{P.D(1),P.D(2),P.D(3),P.D(4),P.D(5),P.D(6); ...
              P.D(2),P.D(7),P.D(8),P.D(9),P.D(10),P.D(11); ...
              P.D(3),P.D(8),P.D(12),P.D(13),P.D(14),P.D(15); ...
              P.D(4),P.D(9),P.D(13),P.D(16),P.D(17),P.D(18); ...
              P.D(5),P.D(10),P.D(14),P.D(17),P.D(19),P.D(20); ...
              P.D(6),P.D(11),P.D(15),P.D(18),P.D(20),P.D(21)}};
elseif (isfield(P,'E') && isfield(P,'nu')) 
    equ.nu = {P.nu};
    equ.E = {P.E};
else
    error('No stiffness type specified');
end
equ.ind = 1;
appl.equ = equ;

fem.appl{1} = appl;
fem.border = 1;
fem=multiphysics(fem);

% ODE Settings
clear ode
clear units;
units.basesystem = 'SI';
ode.units = units;
fem.ode = ode;

fem = multiphysics(fem);

%% Solve with best mesh possible
ok = 0;
if useSym(3)
    botBnd = bndindex(fem.geom,[0,0,0],[0,0,1]);
else
    botBnd = bndindex(fem.geom,[0,0,-thi/2],[0,0,1]);
end
topBnd = bndindex(fem.geom,[0,0,thi/2],[0,0,1]);
display(['Meshing with quality: ',num2str(meshQuality)]);
if meshSweep
    fem.mesh=meshsweep(fem,'hauto',meshQuality,'hmax',lambdaMesh/lambdaDiv, ...
                       'sourceface',{1,botBnd},'targetface',{1,topBnd});
else
    fem.mesh=meshinit(fem,'hauto',meshQuality,'hmax',lambdaMesh/lambdaDiv);
end
fem.xmesh=meshextend(fem);
dofs = xmesh_dofs(fem);
display(['Estimated # of DoFs: ',num2str(dofs)]);

while ok==0 && meshQuality<10
    while dofs >= maxDoF && meshQuality < 10
        % Initialize mesh
        meshQuality = meshQuality +1;
        display(['Meshing with quality: ',num2str(meshQuality)]);
        if meshSweep
            fem.mesh=meshsweep(fem,'hauto',meshQuality,'hmax',lambdaMesh/lambdaDiv, ...
                               'sourceface',{1,botBnd},'targetface',{1,topBnd});
        else
            fem.mesh=meshinit(fem,'hauto',meshQuality,'hmax',lambdaMesh/lambdaDiv);
        end
        fem.xmesh=meshextend(fem);
        dofs = xmesh_dofs(fem);
        display(['Estimated # of DoFs: ',num2str(dofs)]);
    end
    try
        fem.sol=femeig(fem, ...
                   'conjugate','on', ...
                   'solcomp',{'w','v','u'}, ...
                   'outcomp',{'w','v','u'}, ...
                   'neigs',neigs, ...
                   'linsolver','spooles', ...
                   'shift',-1i*2*pi*freq);
               
        display('Done')
        ok = 1;
        ds.F = abs(fem.sol.lambda)/2/pi;
    catch
        display(lasterr)
        meshQuality = meshQuality +1;
        display(['Meshing with quality: ',num2str(meshQuality)]);
        if meshSweep
            fem.mesh=meshsweep(fem,'hauto',meshQuality,'hmax',lambdaMesh/lambdaDiv, ...
                               'sourceface',{1,botBnd},'targetface',{1,topBnd});
        else
            fem.mesh=meshinit(fem,'hauto',meshQuality,'hmax',lambdaMesh/lambdaDiv);
        end
        fem.xmesh=meshextend(fem);
        dofs = xmesh_dofs(fem);
        display(['Estimated # of DoFs: ',num2str(dofs)]);
    end
end

ds.fem = fem;